// MG4interface.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2016 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Wrappers for the MadGraph F77 matrix elements

#include "Vincia/MG4interface.h"

namespace Vincia {

  // Wrappers for common blocks, ported from coupl.inc
  
  extern "C" {
    
    extern struct {
      double          g;        
      complex<double> gg[2];        
    } coupl_qcd_;

    extern struct {
      double          hmass, wmass, zmass, amass, tmass, bmass, lmass, 
        cmass, dmass, umass, smass;
    } coupl_mass_;
    
    extern struct {
      double          hwidth, wwidth, zwidth, twidth, lwidth, awidth;
    } coupl_width_;
    
    extern struct {
      complex<double> gal[2], gad[2], gau[2], gwf[2], gzn[2], gzl[2], gzd[2], gzu[2];
      double          gw, gwwa, gwwz;      
    } coupl_gauge_;
    
    extern struct {
      complex<double> gwfud[2],  gwfcd[2], gwftd[2],
        gwfus[2],  gwfcs[2], gwfts[2],
        gwfub[2],  gwfcb[2], gwftb[2];
    } coupl_ckm_;
    
    // Uncomment for MadGraph with diagonal CKM:
    /*
    extern struct {
      complex<double> gwfc[2],  gwfs[2], gwfm[2];    
    } coupl_ckm_;
    */

    extern struct {
      complex<double> gwwh, gzzh, gwwhh, gzzhh, ghhh, ghhhh;
    } coupl_scal_;
    
    extern struct {
      complex<double> ghtop[2], ghbot[2], ghcha[2], ghstr[2], ghtau[2];
    } coupl_yuk_;

    //Add coupling of higgs to gluons for PS/ME comparison
    extern struct {
      complex<double> gh[2];
    } coupl_heft_;
      
    // Wrappers for MG decay matrix elements
    
    // Z -> jets

    // Born
    extern void sz2ddx_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void sz2dgdx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void sz2dggdx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sz2duxudx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sz2dsxsdx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sz2ddxddx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void sz2dgggdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sz2duxugdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sz2dsxsgdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sz2ddxdgdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);

    // 4th Order
    extern void sz2dggggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2duxuggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2dsxsggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2ddxdggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2dsxsuxudx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2dsxsbxbdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2duxuuxudx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2dsxssxsdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sz2ddxddxddx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    
    // W -> jets
    
    // Born
    extern void swx2udx_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void swx2ugdx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void swx2uggdx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void swx2uuxudx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void swx2udxddx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void swx2usxsdx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void swx2ugggdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void swx2uuxugdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void swx2udxdgdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void swx2usxsgdx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);

    // 4th Order
    extern void swx2uggggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2uuxuggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2udxdggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2usxsggdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2uuxuuxudx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2udxddxddx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2uuxudxddx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2uuxusxsdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2udxdsxsdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2usxssxsdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void swx2usxscxcdx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    
    // H -> jets

    // Born
    extern void sh2gg_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);
    extern void sh2bbx_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void sh2ggg_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sh2bgbx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void sh2gggg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sh2bggbx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sh2bcxcbx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sh2bbxbbx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void sh2ggggg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sh2bgggbx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sh2bcxcgbx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sh2bbxbgbx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);

    // 4th Order
    extern void sh2gggggg_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bggggbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bcxcggbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bbxbggbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bcxcsxsbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bcxccxcbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sh2bbxbbxbbx_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    
    // pp -> Z + jets

    // Born
    extern void sddx2z_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void sddx2gz_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgd2dz_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgdx2dxz_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void sddx2ggz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgg2ddxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgd2gdz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdd2ddz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdu2duz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sds2dsz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sddx2ddxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdux2duxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdsx2dsxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    //extern void sddx2ddxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sddx2uuxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sddx2ssxz_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void sddx2gggz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2ddxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdg2dggz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdg2dddxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdg2dssxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdg2duuxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxg2dxddxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxg2dxssxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxg2dxuuxz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sddx2ddxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sddx2ssxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sddx2uuxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdd2ddgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sds2dsgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdu2dugz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    //extern void sddx2ddxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdsx2dsxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdux2duxgz_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    
    // pp -> W + jets

    // Born
    extern void sudx2wx_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void sudx2gwx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sug2dwx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgdx2uxwx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void sudx2ggwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sug2gdwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgdx2guxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgg2uxdwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suux2duxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdxux2uxuxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sudx2uuxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suu2udwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sddx2duxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdxdx2dxuxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdxu2ddxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sdu2ddwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void scxdx2cxuxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sucx2dcxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void scdx2cuxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void scu2cdwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sudx2ccxwx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void sudx2gggwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2ggdwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgdx2gguxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2guxdwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2ccxdwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxu2ccxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxg2ccxuxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scxu2cxdgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scxg2cxduxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scxdx2cxuxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scu2cdgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scg2cduxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void scdx2cuxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sddx2duxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdg2dduxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdu2ddgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxdx2dxuxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxg2ddxuxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2ddxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2dddxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sdxux2uxuxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgdx2uuxuxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgux2uxuxdwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2uuxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2duuxwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suu2dugwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2duxgwx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    
    // pp -> H + jets

    // Born
    extern void sgg2h_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);
    extern void sbbx2h_(int *cMode, int *ICOL, double P1[3][4], int HEL[4], double ANS[1]);

    // 1st Order
    extern void sgg2gh_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgb2bh_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgbx2bxh_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sbbx2gh_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);

    // 2nd Order
    extern void sgg2ggh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgg2bbxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgb2bgh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgbx2bxgh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbbx2ggh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbb2bbh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbbx2bbxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbxbx2bxbxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbc2bch_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbcx2bcxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbxcx2bxcxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sbbx2ccxh_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 3rd Order
    extern void sgg2gggh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2bbxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgb2bggh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgbx2bxggh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbbx2gggh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbb2bbgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbbx2bbxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbxbx2bxbxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbg2bbbxh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbxg2bxbbxh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbc2bcgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbcx2bcxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbxcx2bxcxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbbx2ccxgh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbg2bccxh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sbxg2bxccxh_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    
    // QCD 2->n

    // Born
    extern void sgg2gg_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sgg2uux_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sug2ug_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void suux2gg_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void suux2uux_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void suu2uu_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void suux2ddx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sud2ud_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    extern void sudx2udx_(int *cMode, int *ICOL, double P1[4][4], int HEL[4], double ANS[1]);
    
    // 1st order
    extern void sgg2ggg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sgg2uuxg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sug2ugg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suux2ggg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suux2uuxg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suu2uug_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sug2uuux_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void suux2ddxg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sud2udg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sudx2udxg_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);
    extern void sug2uddx_(int *cMode, int *ICOL, double P1[5][4], int HEL[4], double ANS[1]);

    // 2nd order
    extern void sgg2gggg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2uuxgg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2uggg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2gggg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2uuxgg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suu2uugg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2uuuxg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2uuxuux_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2ddxgg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sud2udgg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2udxgg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sug2uddxg_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sgg2uuxddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2uuxuux_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suu2uuuux_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2ddxddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2uuxddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suu2uuddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sud2udddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2udxddx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2udxuux_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void suux2ddxssx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sudx2udxssx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);
    extern void sud2udssx_(int *cMode, int *ICOL, double P1[6][4], int HEL[4], double ANS[1]);

    // 3rd order
    //extern void sgg2ggggg_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sgg2uuxggg_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void sug2ugggg_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);
    extern void suux2ggggg_(int *cMode, int *ICOL, double P1[7][4], int HEL[4], double ANS[1]);

  }

  //*********

  // Function to swap parameters (eg to use Z->bb matrix element for Z->uu)
  // IMPORTANT NOTE: always remember to swap back after such operations

  void MG4interface::swapZud() {
    complex<double> tmp0 = coupl_gauge_.gzu[0];
    complex<double> tmp1 = coupl_gauge_.gzu[1];
    coupl_gauge_.gzu[0] = coupl_gauge_.gzd[0];
    coupl_gauge_.gzu[1] = coupl_gauge_.gzd[1];
    coupl_gauge_.gzd[0] = tmp0;
    coupl_gauge_.gzd[1] = tmp1;
  }

  //*********
    
  // Function to set the coupling for W
  
  void MG4interface::setWcouplings(int id1, int id2) {
    // MEs are using gwf
    if (id1 > id2) {
      int idTmp = id1;
      id1 = id2;
      id2 = idTmp;
    }
    if (id1 == 1) {
      if (id2 == 2) {
        coupl_gauge_.gwf[0] = Vud * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vud * complex<double>( gwf2 , 0.0 );
      } else if (id2 == 4) {
        coupl_gauge_.gwf[0] = Vcd * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vcd * complex<double>( gwf2 , 0.0 );
      } else if (id2 == 6) {
        coupl_gauge_.gwf[0] = Vtd * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vtd * complex<double>( gwf2 , 0.0 );
      }
    } else if (id1 == 2) {
      if (id2 == 3) {
        coupl_gauge_.gwf[0] = Vus * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vus * complex<double>( gwf2 , 0.0 );
      } else if (id2 == 5) {
        coupl_gauge_.gwf[0] = Vub * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vub * complex<double>( gwf2 , 0.0 );
      }
    } else if (id1 == 3) {
      if (id2 == 4) {
        coupl_gauge_.gwf[0] = Vcs * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vcs * complex<double>( gwf2 , 0.0 );
      } else if (id2 == 6) {
        coupl_gauge_.gwf[0] = Vts * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vts * complex<double>( gwf2 , 0.0 );
      }
    } else if (id1 == 4) {
      if (id2 == 5) {
        coupl_gauge_.gwf[0] = Vcb * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vcb * complex<double>( gwf2 , 0.0 );
      }
    } else if (id1 == 5) {
      if (id2 == 6) {
        coupl_gauge_.gwf[0] = Vtb * complex<double>( gwf1 , 0.0 );
        coupl_gauge_.gwf[1] = Vtb * complex<double>( gwf2 , 0.0 );
      }
    }
  }

  //*********
  
  // Functions to set order for q qbar' q' qbar'' q'' qbar with one distinct pair
  vector<int> MG4interface::orderQQBqqbqqb(pair<int,int> posQQB, bool firstLast = true) {
   vector<int> order;
   order.clear(); 
   order.resize(0);
   order.push_back(0);
   if (firstLast) {
     order.push_back(posQQB.first);
     order.push_back(posQQB.first+1);
   }
        if (posQQB == make_pair(1,4)) { order.push_back(5); order.push_back(6); }
   else if (posQQB == make_pair(1,6)) { order.push_back(3); order.push_back(4); }
   else if (posQQB == make_pair(3,2)) { order.push_back(5); order.push_back(6); }
   else if (posQQB == make_pair(3,6)) { order.push_back(1); order.push_back(2); }
   else if (posQQB == make_pair(5,2)) { order.push_back(3); order.push_back(4); }
   else if (posQQB == make_pair(5,4)) { order.push_back(1); order.push_back(2); }
   order.push_back(posQQB.second-1);
   order.push_back(posQQB.second);
   if (!firstLast) {
     order.push_back(posQQB.first);
     order.push_back(posQQB.first+1);
   }
   return order;
  }
  vector<int> MG4interface::orderQQBqqbqqbQCD(pair<int,int> posQQB, bool firstLast = true) {
   vector<int> order;
   order.clear(); 
   order.resize(0);
   if (firstLast) {
     order.push_back(posQQB.first);
     order.push_back(posQQB.first+1);
   }
        if (posQQB == make_pair(0,3)) { order.push_back(4); order.push_back(5); }
   else if (posQQB == make_pair(0,5)) { order.push_back(2); order.push_back(3); }
   else if (posQQB == make_pair(2,1)) { order.push_back(4); order.push_back(5); }
   else if (posQQB == make_pair(2,5)) { order.push_back(0); order.push_back(1); }
   else if (posQQB == make_pair(4,1)) { order.push_back(2); order.push_back(3); }
   else if (posQQB == make_pair(4,3)) { order.push_back(0); order.push_back(1); }
   order.push_back(posQQB.second-1);
   order.push_back(posQQB.second);
   if (!firstLast) {
     order.push_back(posQQB.first);
     order.push_back(posQQB.first+1);
   }
   return order;
  }

  //*********
    
  // Functions to get order for W decays
  vector<int> MG4interface::orderWfourQuarks(vector<int> ids) {
    vector<int> order;
    order.push_back(0);
    // q qbar' q' qbar''
    if (abs(ids[2]) == abs(ids[3])) {
      for (int i=1; i<=4; i++) order.push_back(i);
    } else {
      order.push_back(3);
      order.push_back(4);
      order.push_back(1);
      order.push_back(2);
    }
    return order;
  }
  vector<int> MG4interface::orderWfourQuarksG(vector<int> ids, bool& gSwap) {
    vector<int> order;
    order.push_back(0);
    gSwap = false;
    // q qbar' q' g qbar''
    if (abs(ids[2]) == abs(ids[3])) {
      for (int i=1; i<=5; i++) order.push_back(i);
    } else {
      order.push_back(3);
      order.push_back(5);
      order.push_back(1);
      order.push_back(4);
      order.push_back(2);
      gSwap = true;
    }
    return order;
  }
  vector<int> MG4interface::orderWfourQuarksGG(vector<int> ids, bool& gg, bool& gSwap) {
    vector<int> order;
    order.push_back(0);
    gg    = false;
    gSwap = false;
    // q qbar' q' g g qbar''
    if (ids[4] == 21 && ids[5] == 21) {
      gg = true;
      if (abs(ids[2]) == abs(ids[3])) {
        for (int i=1; i<=6; i++) order.push_back(i);
      } else {
        order.push_back(3);
        order.push_back(6);
        order.push_back(1);
        order.push_back(4);
        order.push_back(5);
        order.push_back(2);
        gSwap = true;
      }
    } 
    // q g qbar' q' g qbar''
    else {
      gg = false;
      if (abs(ids[3]) == abs(ids[4])) {
        order.push_back(1);
        order.push_back(3);
        order.push_back(4);
        order.push_back(2);
        order.push_back(5);
        order.push_back(6);
      } else {
        order.push_back(4);
        order.push_back(6);
        order.push_back(1);
        order.push_back(5);
        order.push_back(2);
        order.push_back(3);
      }
    }
    return order;
  }
  vector<int> MG4interface::orderWsixQuarks(vector<int> ids, map<int,int>& flavours,
                                            bool& flav2ordChange) {
    // q qbar q qbar q qbar
    flavours.clear();
    flav2ordChange = false;
    for (int i=1; i<=5; ++i) flavours[i] = 0;
    for (int i=1; i<(int)ids.size(); ++i) flavours[abs(ids[i])]++;
    int nrDiffFlavs = 0;
    for (int i=1; i<=5; ++i) if (flavours[i] > 0) nrDiffFlavs++;
    if (nrDiffFlavs == 2) {
      // If we have 1 + 5 combi: q qbar q qbar q qbar'
      //                         q' qbar q qbar q qbar
      for (int i=1; i<=5; ++i) if (flavours[i] == 1) {
        int flav = i;
        int iQ1  = -1;
        int iQ2  = -1;
        for (int j=1; j<(int)ids.size(); ++j) if (abs(ids[j]) == flav) iQ1 = j;
        iQ2 = (iQ1 <= 3 ? iQ1+3 : iQ1-3);
        if (iQ2 % 2) { // iQ2 is q, but needs to be qbar
          int iQtmp = iQ1;
          iQ1 = iQ2;
          iQ2 = iQtmp;
        }
        return orderQQBqqbqqb(make_pair(iQ1, iQ2), true);
      }
      // If we have 3 + 3 combi
      int nrDiffFlavQP = 0;
      for (int i=1; i<6; i+=2) if (abs(ids[i]) != abs(ids[i+1])) nrDiffFlavQP++;
      if (nrDiffFlavQP == 1) { // q qbar q qbar' q' qbar'
        if (abs(ids[1]) != abs(ids[2]))
          return orderQQBqqbqqb((ids[1] == ids[3] ? make_pair(3, 6) : make_pair(5, 4)), true);
        if (abs(ids[3]) != abs(ids[4]))
          return orderQQBqqbqqb((ids[1] == ids[3] ? make_pair(1, 6) : make_pair(5, 2)), true);
        if (abs(ids[5]) != abs(ids[6]))
          return orderQQBqqbqqb((ids[1] == ids[5] ? make_pair(1, 4) : make_pair(3, 2)), true);
      }
      vector<int> order;
      order.push_back(0);
      if ( (abs(ids[1]) == abs(ids[4])) && (abs(ids[1]) == abs(ids[5])) ) { // q qbar' q' qbar q qbar'
        order.push_back(1);
        order.push_back(4);
        order.push_back(5);
        order.push_back(2);
        order.push_back(3);
        order.push_back(6);
      } else if ( (abs(ids[1]) == abs(ids[3])) && (abs(ids[1]) == abs(ids[6])) ) { // q qbar' q qbar' q' qbar
        order.push_back(1);
        order.push_back(6);
        order.push_back(3);
        order.push_back(2);
        order.push_back(5);
        order.push_back(4);
      } else if ( (abs(ids[1]) == abs(ids[4])) && (abs(ids[1]) == abs(ids[6])) ) { // q' qbar q qbar' q qbar'
        order.push_back(3);
        order.push_back(2);
        order.push_back(5);
        order.push_back(4);
        order.push_back(1);
        order.push_back(6);
      }
      flav2ordChange = true;
      if (flav2ordChange) return order;
    }
    if (nrDiffFlavs == 3) {
      bool standardProd = true;
      // If emitted quark pairs have the same flavour, do standard procedure
      for (int i=1; i<=5; ++i) if (flavours[i] == 3) standardProd = false;
      if (!standardProd) { // q qbar q qbar' q' qbar''
                           // q qbar' q' qbar q qbar''
                           // q'' qbar' q' qbar q qbar
                           // q'' qbar q qbar' q' qbar
        // Find quark flavours to look for
        int flav1 = 0;
        int flav2 = 0;
        for (int i=1; i<=5; ++i) {
          if (flavours[i] == 1) flav1 = i;
          else if ( (flavours[i] % 2) && (flavours[i] > 1) ) flav2 = i;
        }
        // Find first quark
        int iQ1 = -1;
        for (int i=1; i<(int)ids.size(); ++i) if (abs(ids[i]) == flav1) {
          iQ1 = i;
          break;
        }
        // Find second quark
        int iQ2 = -1;
        for (int i=1; i<(int)ids.size(); ++i) if (abs(ids[i]) == flav2) {
          // iQ1 is q
          if ((iQ1 % 2) && !(i % 2) && (i != iQ1+1)) iQ2 = i;
          // iQ1 is qbar
          else if (!(iQ1 % 2) && (i % 2) && (i != iQ1-1)) iQ2 = i;
          if (iQ2 > 0) break;
        }
        if (iQ2 % 2) { // iQ2 is q, but needs to be qbar
          int iQtmp = iQ1;
          iQ1 = iQ2;
          iQ2 = iQtmp;
        }
        return orderQQBqqbqqb(make_pair(iQ1, iQ2), true);
      }
    }
    // Standard procedure for: q'' qbar q qbar' q' qbar'''
    //                         q' qbar q qbar q qbar''
    vector<int> flavs;
    for (int i=1; i<=5; ++i) // 2 odd appearences
      if (flavours[i] % 2) flavs.push_back(i);
    int iQ    = -1;
    int iQbar = -1;
    for (int i=1; i<(int)ids.size(); ++i) {
      for (int j=0; j<2; j++) {
        if (abs(ids[i]) == flavs[j]) {
          if (ids[i] > 0) iQ    = i;
          else            iQbar = i;
        }
      }
    }
    return orderQQBqqbqqb(make_pair(iQ, iQbar), true);
  }

  //*********
    
  // Function to set the Yukawa couplings and masses
  void MG4interface::setYukawa(int id1, double m1) {
    coupl_mass_.bmass   = m1;
    double yukCoupNow1  = ( ((m1>0.0) && (abs(yukCoup[abs(id1)])>0.0)) ? (-m1/vev) : yukCoup[abs(id1)] );
    coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoupNow1 );
    coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoupNow1 );
  }
  void MG4interface::setYukawa(int id1, int id2, double m1, double m2) {
    coupl_mass_.bmass   = m1;
    coupl_mass_.cmass   = m2;
    double yukCoupNow1  = ( ((m1>0.0) && (abs(yukCoup[abs(id1)])>0.0)) ? (-m1/vev) : yukCoup[abs(id1)] );
    double yukCoupNow2  = ( ((m2>0.0) && (abs(yukCoup[abs(id2)])>0.0)) ? (-m2/vev) : yukCoup[abs(id2)] );
    coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoupNow1 );
    coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoupNow1 );
    coupl_yuk_.ghcha[0] = complex<double>( 0.0, yukCoupNow2 );
    coupl_yuk_.ghcha[1] = complex<double>( 0.0, yukCoupNow2 );
  }
  void MG4interface::setYukawa(int id1, int id2, int id3,
                               double m1, double m2, double m3) {
    coupl_mass_.bmass   = m1;
    coupl_mass_.cmass   = m2;
    coupl_mass_.smass   = m3;
    double yukCoupNow1  = ( ((m1>0.0) && (abs(yukCoup[abs(id1)])>0.0)) ? (-m1/vev) : yukCoup[abs(id1)] );
    double yukCoupNow2  = ( ((m2>0.0) && (abs(yukCoup[abs(id2)])>0.0)) ? (-m2/vev) : yukCoup[abs(id2)] );
    double yukCoupNow3  = ( ((m3>0.0) && (abs(yukCoup[abs(id3)])>0.0)) ? (-m3/vev) : yukCoup[abs(id3)] );
    coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoupNow1 );
    coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoupNow1 );
    coupl_yuk_.ghcha[0] = complex<double>( 0.0, yukCoupNow2 );
    coupl_yuk_.ghcha[1] = complex<double>( 0.0, yukCoupNow2 );
    coupl_yuk_.ghstr[0] = complex<double>( 0.0, yukCoupNow3 );
    coupl_yuk_.ghstr[1] = complex<double>( 0.0, yukCoupNow3 );
  }
    
  // Function to set either Higgs-g for Higgs-q coupling to zero
  void MG4interface::setHinterferenceOff(bool heftIn) {
    if (heftIn) {
      // Zero heft coupling
      coupl_heft_.gh[0] = complex<double>( 0.0, 0.0 );
      coupl_heft_.gh[1] = complex<double>( 0.0, 0.0 );
      // Recover Yukawa couplings
      yukCoup           = yukCoupSav;
    } else {
      // Recover heft coupling
      coupl_heft_.gh[0] = complex<double>( 0.0, heftCoupSav );
      coupl_heft_.gh[1] = complex<double>( 0.0, heftCoupSav );
      // Zero Yukawa couplings
      for (int i=0; i<(int)yukCoup.size(); i++) yukCoup[i] = 0.0;
    }
  }

  //*********

  // Initialize MadGraph
  // Function to initialize the MadGraph coupling common blocks, required
  // by the MG matrix elements. Adapted from couplings.f in the MG 
  // Models/sm/ directory.

  bool MG4interface::init() {
    
    // Set verbose level
    verbose = settingsPtr->mode("Vincia:verbose");

    // Verbose output
    if (verbose >= 2) printOut("MG4interface::init","...");

    // Check if pointers initialized
    if (! isInitPtr) {
      printOut("MG4interface::init","Cannot initialize, pointers not set.");
      return false;      
    }
    
    // Innocent until proven guilty
    isInit = true;

    // Verbose output
    if (verbose >= 3) printOut("MG4interface::init","   setting MG masses...");

    // Set masses and widths (initialized through Pythia)
    coupl_mass_.dmass   = particleDataPtr->m0(1);
    coupl_mass_.umass   = particleDataPtr->m0(2);
    coupl_mass_.smass   = particleDataPtr->m0(3);
    coupl_mass_.cmass   = particleDataPtr->m0(4);
    coupl_mass_.bmass   = particleDataPtr->m0(5);
    coupl_mass_.tmass   = particleDataPtr->m0(6);
    coupl_mass_.lmass   = particleDataPtr->m0(15);
    coupl_mass_.amass   = particleDataPtr->m0(22);
    coupl_mass_.zmass   = particleDataPtr->m0(23);
    coupl_mass_.wmass   = particleDataPtr->m0(24);
    coupl_mass_.hmass   = particleDataPtr->m0(25);
    coupl_width_.twidth = particleDataPtr->mWidth(6);
    coupl_width_.lwidth = particleDataPtr->mWidth(15);
    coupl_width_.awidth = particleDataPtr->mWidth(22);
    coupl_width_.zwidth = particleDataPtr->mWidth(23);
    coupl_width_.wwidth = particleDataPtr->mWidth(24);
    coupl_width_.hwidth = particleDataPtr->mWidth(25);

    // Useful shorthands
    const double Fourpi = 4.0 * M_PI;
    const double Zero   = 0.0;
    // Here using alpha(mZ) (as in SLHA) rather than alpha(0)
    // Check consistency when doing full matrix elements.
    const double alpha  = settingsPtr->parm("StandardModel:alphaEMmZ");
    // Use thetabar for VFF couplings? (as in Pythia)
    // Check consistency when doing full matrix elements.
    const double sin2w  = couplingsPtr->sin2thetaW();
    const double cw     = sqrt( 1.0 - sin2w );
    const double ee2    = alpha * Fourpi;
    const double sw     = sqrt( sin2w );
    const double ee     = sqrt( ee2 );
    const double ez     = ee/(sw*cw);
    const double ey     = ee*(sw/cw);
    const double sc2    = sin2w*( 1.0 - sin2w );
    const double v      = 2.0*coupl_mass_.wmass*sw/ee;    
    //const double lambda = pow2(coupl_mass_.hmass) / (2.0 * pow2(v));
    // CKM
    Vud = couplingsPtr->VCKMgen(1,1);
    Vus = couplingsPtr->VCKMgen(1,2);
    Vub = couplingsPtr->VCKMgen(1,3);
    Vcd = couplingsPtr->VCKMgen(2,1);
    Vcs = couplingsPtr->VCKMgen(2,2);
    Vcb = couplingsPtr->VCKMgen(2,3);
    Vtd = couplingsPtr->VCKMgen(3,1);
    Vts = couplingsPtr->VCKMgen(3,2);
    Vtb = couplingsPtr->VCKMgen(3,3);

    // Verbose output
    if (verbose >= 3) printOut("MG4interface::init","   setting MG couplings...");

    // Set QCD g = 1 to keep factored out (=> alphaS = 1/sqrt(4pi))
    coupl_qcd_.g      = 1.0;
    coupl_qcd_.gg[0]  = -coupl_qcd_.g;
    coupl_qcd_.gg[1]  = -coupl_qcd_.g;

    // Set Vector boson couplings
    coupl_gauge_.gw   = ee/sw;
    coupl_gauge_.gwwa = ee;
    coupl_gauge_.gwwz = ee*cw/sw;
    
    // Set gauge & higgs boson couplings
    coupl_scal_.gwwh  = complex<double>( ee2/sin2w*0.5*v, Zero );
    coupl_scal_.gzzh  = complex<double>( ee2/sc2*0.5*v  , Zero );
    coupl_scal_.ghhh  = complex<double>( -pow2(coupl_mass_.hmass)/v*3.0, Zero );
    coupl_scal_.gwwhh = complex<double>( ee2/sin2w*0.5  , Zero );
    coupl_scal_.gzzhh = complex<double>( ee2/sc2*0.5    , Zero );
    coupl_scal_.ghhhh = coupl_scal_.ghhh/v;

    // Set fermion-fermion-vector couplings
    coupl_gauge_.gal[0] = complex<double>(  ee          , Zero );
    coupl_gauge_.gal[1] = complex<double>(  ee          , Zero );
    coupl_gauge_.gau[0] = complex<double>( -ee*2./3.    , Zero );
    coupl_gauge_.gau[1] = complex<double>( -ee*2./3.    , Zero );
    coupl_gauge_.gad[0] = complex<double>(  ee/3.       , Zero );
    coupl_gauge_.gad[1] = complex<double>(  ee/3.       , Zero );

    coupl_gauge_.gzn[0] = complex<double>( -ez*0.5                  , Zero );
    coupl_gauge_.gzn[1] = complex<double>(  Zero                    , Zero );
    coupl_gauge_.gzl[0] = complex<double>( -ez*(-0.5 + sin2w)       , Zero );
    coupl_gauge_.gzl[1] = complex<double>( -ey                      , Zero );
    coupl_gauge_.gzu[0] = complex<double>( -ez*( 0.5 - sin2w*2./3.) , Zero );
    coupl_gauge_.gzu[1] = complex<double>(  ey*2./3.                , Zero );
    coupl_gauge_.gzd[0] = complex<double>( -ez*(-0.5 + sin2w/3.)    , Zero );
    coupl_gauge_.gzd[1] = complex<double>( -ey/3.                   , Zero );

    gwf1 = -ee/sqrt(2.*sin2w);
    gwf2 = Zero;
    coupl_gauge_.gwf[0] = complex<double>( gwf1 , Zero );
    coupl_gauge_.gwf[1] = complex<double>( gwf2 , Zero );

    // CKM
    for (int iLR = 0; iLR <= 1; ++iLR) {
      coupl_ckm_.gwfud[iLR] = Vud * coupl_gauge_.gwf[iLR]; 
      coupl_ckm_.gwfus[iLR] = Vus * coupl_gauge_.gwf[iLR];
      coupl_ckm_.gwfub[iLR] = Vub * coupl_gauge_.gwf[iLR];
      coupl_ckm_.gwfcd[iLR] = Vcd * coupl_gauge_.gwf[iLR]; 
      coupl_ckm_.gwfcs[iLR] = Vcs * coupl_gauge_.gwf[iLR];
      coupl_ckm_.gwfcb[iLR] = Vcb * coupl_gauge_.gwf[iLR];
      coupl_ckm_.gwftd[iLR] = Vtd * coupl_gauge_.gwf[iLR]; 
      coupl_ckm_.gwfts[iLR] = Vts * coupl_gauge_.gwf[iLR];
      coupl_ckm_.gwftb[iLR] = Vtb * coupl_gauge_.gwf[iLR];
   }

    // Set the Yukawa couplings for the different flavours
    // This is used for H production (H decay uses actual quark masses if > 0)
    // Make sure the mass is not set to zero
    vev = v;
    yukCoup.clear();
    yukCoup.resize(0);
    yukCoup.push_back(0.0);
    if (particleDataPtr->m0(1) < TINY) yukCoup.push_back(-0.005/vev);
    else yukCoup.push_back(-particleDataPtr->m0(1)/vev);
    if (particleDataPtr->m0(1) < TINY) yukCoup.push_back(-0.003/vev);
    else yukCoup.push_back(-particleDataPtr->m0(2)/vev);
    if (particleDataPtr->m0(3) < TINY) yukCoup.push_back(-0.1/vev);
    else yukCoup.push_back(-particleDataPtr->m0(3)/vev);
    if (particleDataPtr->m0(4) < TINY) yukCoup.push_back(-1.5/vev);
    else yukCoup.push_back(-particleDataPtr->m0(4)/vev);
    if (particleDataPtr->m0(5) < TINY) yukCoup.push_back(-4.8/vev);
    else yukCoup.push_back(-particleDataPtr->m0(5)/vev);
    coupl_yuk_.ghbot[0] = complex<double>( Zero, yukCoup[5] );
    coupl_yuk_.ghbot[1] = complex<double>( Zero, yukCoup[5] );
    coupl_yuk_.ghcha[0] = complex<double>( Zero, yukCoup[4] );
    coupl_yuk_.ghcha[1] = complex<double>( Zero, yukCoup[4] );
    coupl_yuk_.ghstr[0] = complex<double>( Zero, yukCoup[3] );
    coupl_yuk_.ghstr[1] = complex<double>( Zero, yukCoup[3] );
    yukCoupSav = yukCoup;
    
    // TODO At the moment we don't include the running
    // Include coupling of Higgs to gluons
    // NOTE value taken from MadGraph ( = alphaS/3/pi/vev(1+corr) )
    heftCoupSav = 0.000054936089955251788;
    coupl_heft_.gh[0] = complex<double>( Zero, heftCoupSav );
    coupl_heft_.gh[1] = complex<double>( Zero, heftCoupSav );

    // Verbose output
    if (verbose >= 3) printOut("MG4interface::init","   setting Color Depth...");

    // Set color depth
    colorDepth = (settingsPtr->flag("Vincia:matchingFullColor")) ? 1 : 0;

    return true;
  }
  
  //*********

  // Wrapper to set values in the MG common blocks

  void MG4interface::setParm(string txt, complex<double> val) {

    // Check if initialized
    if (! isInit) {
      printOut("MG4interface::setParm","MG4interface not initialized.");
    }
    // Convert to all lower-case
    for (unsigned int i=0;i<txt.length();++i) txt[i]=tolower(txt[i]);
    if (txt == "zmass" || txt == "mz") coupl_mass_.zmass   = val.real();
    else if (txt == "wmass" || txt == "mw") coupl_mass_.wmass   = val.real();
    else if (txt == "bmass" || txt == "mb") coupl_mass_.bmass   = val.real();
    else if (txt == "hmass" || txt == "mh") coupl_mass_.hmass   = val.real();
    // A (photon) couplings
    else if (txt == "gau0") coupl_gauge_.gau[0]=val;
    else if (txt == "gau1") coupl_gauge_.gau[1]=val;
    else if (txt == "gad0") coupl_gauge_.gad[0]=val;
    else if (txt == "gad1") coupl_gauge_.gad[1]=val;
    else if (txt == "gal0") coupl_gauge_.gal[0]=val;
    else if (txt == "gal1") coupl_gauge_.gal[1]=val;
    // Z couplings
    else if (txt == "gzu0") coupl_gauge_.gzu[0]=val;
    else if (txt == "gzu1") coupl_gauge_.gzu[1]=val;
    else if (txt == "gzd0") coupl_gauge_.gzd[0]=val;
    else if (txt == "gzd1") coupl_gauge_.gzd[1]=val;
    else if (txt == "gzn0") coupl_gauge_.gzn[0]=val;
    else if (txt == "gzn1") coupl_gauge_.gzn[1]=val;
    else if (txt == "gzl0") coupl_gauge_.gzl[0]=val;
    else if (txt == "gzl1") coupl_gauge_.gzl[1]=val;
    // W couplings
    else if (txt.find("gwf") != string::npos) {
      if (txt == "gwf0") coupl_gauge_.gwf[0]=val;
      else if (txt == "gwf1") coupl_gauge_.gwf[1]=val;
      // CKM
      else if (txt == "gwfud0") coupl_ckm_.gwfud[0]=val;
      else if (txt == "gwfud1") coupl_ckm_.gwfud[1]=val;
      else if (txt == "gwfcd0") coupl_ckm_.gwfcd[0]=val;
      else if (txt == "gwfcd1") coupl_ckm_.gwfcd[1]=val;
      else if (txt == "gwftd0") coupl_ckm_.gwftd[0]=val;
      else if (txt == "gwftd1") coupl_ckm_.gwftd[1]=val;
      else if (txt == "gwfus0") coupl_ckm_.gwfus[0]=val;
      else if (txt == "gwfus1") coupl_ckm_.gwfus[1]=val;
      else if (txt == "gwfcs0") coupl_ckm_.gwfcs[0]=val;
      else if (txt == "gwfcs1") coupl_ckm_.gwfcs[1]=val;
      else if (txt == "gwfts0") coupl_ckm_.gwfts[0]=val;
      else if (txt == "gwfts1") coupl_ckm_.gwfts[1]=val;
      else if (txt == "gwfub0") coupl_ckm_.gwfub[0]=val;
      else if (txt == "gwfub1") coupl_ckm_.gwfub[1]=val;
      else if (txt == "gwfcb0") coupl_ckm_.gwfcb[0]=val;
      else if (txt == "gwfcb1") coupl_ckm_.gwfcb[1]=val;
      else if (txt == "gwftb0") coupl_ckm_.gwftb[0]=val;
      else if (txt == "gwftb1") coupl_ckm_.gwftb[1]=val;
    }
    // H couplings
    else if (txt == "ghbot0") coupl_yuk_.ghbot[0] = val;
    else if (txt == "ghbot1") coupl_yuk_.ghbot[1] = val;
    else if (txt == "gh0") coupl_heft_.gh[0] = val;
    else if (txt == "gh1") coupl_heft_.gh[1] = val;
    // Else issue error message: unrecognized coupling parameter
    else {
      printOut("MG4interface::setParm","Unknown parameter "+txt);
    }

  }

  void MG4interface::setParm(string txt, double val) {

    complex<double> valC(val,0.0);
    setParm(txt, valC);

  }

  //*********

  // Wrapper to return unpolarized Madgraph Matrix Elements (squared)
  double MG4interface::ME2(int nIn, vector<int> id, vector<Vec4> p) {

    vector<double> pol;
    for (unsigned int i=0; i<p.size(); ++i) pol.push_back(9.);
    return ME2(nIn, id, p, pol);

  }

  // Wrapper to return polarized MadGraph Matrix Elements (squared)
  // Input:
  //    nIn   = number of particles that are incoming (1 or 2)
  //    id    = vector of particle IDs. The incoming particle(s) MUST be the 
  //            first nIn entries of this vector. The outgoing particles
  //            MUST be arranged in color order thereafter. 
  //    p     = vector of particle momenta. The incoming particle(s) MUST be 
  //            the first nIn entries of this vector. The outgoing particles
  //            MUST be arranged in color order thereafter. 
  //        pol   = vector of particle polarizations.  pol = 9 is unpolarized.
  //                The incoming particle(s) MUST be the first nIn entries of 
  //            this vector. The outgoing particles MUST be arranged in 
  //                color order thereafter.

  // Return:   value of Matrix Element squared for this phase-space point

  double MG4interface::ME2(int nIn, vector<int> id, vector<Vec4> p, 
                          vector<int> pol) {

    // Check if initialized
    if (! isInit) {
      printOut("MG4interface::setParm","MG4interface not initialized.");
    }
    
    // Initialize ME weight with dummy value.
    const  int nMom  = p.size();
    const  int nOut  = nMom-nIn;
    double MGME2 = -1.;

    // Sanity checks
    if (nMom <= 2) return MGME2;
    if (int(id.size()) != nMom) return MGME2;

    // Take out polarization averages
    int nPolIn = 1;
    
    // Default color ordering is number 1
    int icol = 1;

    // HEL allows to specify up to 4 specific helicity amplitudes to sum over
    // (enough for 2 unpolarised incoming particles, or unpolarised decay) 
    // (enumeration follows MADGRAPH IHELs)
    // (To get the full helicity sum, just leave first component = -1)
    // Initialise for full helicity sum (individual IHELs are always positive)
    int HEL[4] = {-1, -1, -1, -1};      
      
    //------------------------------------------------------------------------

    int id0 = abs(id[0]);
    // If not known, return.
    if (id0 == 0) return -1;

    // Check if using massive matrix elements and what kind of ME is asked for    
    vector<double> m;    
    bool massless    = true;
    bool isQCD       = true;
    bool isPolarised = false;
    int nQuarkOut = 0;
    int nGluonOut = 0;
    int nQuarkIn  = 0;
    int nGluonIn  = 0;
    int nPolarised = 0;
    vector<int> iQuark;
    vector<int> iGluon;
    double sCM    = 0.0;
    // Decay processes
    if (nIn == 1) {
      isQCD = false;
      sCM = p[0].m2Calc();
      // Fill vector of desired masses (zero ones that are offshell only by
      // an amount consistent with numerical precision)
      m.push_back(sqrt(p[0].m2Calc()));
      // Also count how many particles have assigned helicities
      if (pol[0] != 9) ++nPolarised;
      for (unsigned int i=1;i<id.size(); ++i) {      
        if (pol[i] != 9) ++nPolarised;
        // Count number of quarks (+antiquarks) and gluons
        if (id[i] == 21) {
          nGluonOut++;
          iGluon.push_back(i);
        }        
        else if (abs(id[i]) <= 10) {
          nQuarkOut++;
          iQuark.push_back(i);
        }
        double m2 = p[i].m2Calc();      
        if (abs(m2/sCM) > TINY) {
          massless = false;
          m.push_back(sqrt(abs(m2)));
        } else {
          m.push_back(0.0);
        }
      }
    }
    // 2->n processes
    else {
      massless = true; // ISR massless!
      sCM = 4.0*p[pos1Sav].e()*p[pos2Sav].e();
      for (int i=0;i<(int)id.size(); ++i) {
        // Count number of particles have assigned helicities
        if (pol[i] != 9) ++nPolarised;
        // Count number of quarks (+antiquarks) and gluons
        if (id[i] == 21) {
          if (i==pos1Sav || i==pos2Sav) nGluonIn++;
          else nGluonOut++;
          iGluon.push_back(i);
        }        
        else if (abs(id[i]) <= 10) {
          if (i==pos1Sav || i==pos2Sav) nQuarkIn++;
          else nQuarkOut++;
          iQuark.push_back(i);
        }
        // Store mass values
        m.push_back(sqrt(p[i].m2Calc()));
        // Check if this is a pure-QCD process
        if (abs(id[i]) > 10 && abs(id[i]) != 21) isQCD = false;
      }
    }
    int nGluonTot = nGluonOut + nGluonIn;
    int nQuarkTot = nQuarkOut + nQuarkIn;
    
    // Check if we should evaluate helicity sum or specified helicity(ies)
    if (nPolarised >= 1 && nMom - nPolarised <= 2) isPolarised = true;
    
    // Verbose output
    if (verbose >= 8) 
      printOut("MG4interface::ME2","massless approximation = "
               +bool2str(massless));

    // Check Lorentz invariance 
    // (change to (true) to always boost back to CM) 
    if (false) {
      Vec4 vCM;
      if (nIn == 1) vCM = p[0];
      else vCM = p[pos1Sav]+p[pos2Sav];
      double eCM = vCM.e();
      if (eCM <= 0.0) {
        printOut("MG4interface::ME2",
                 "Error: failed to evaluate ME, eCM <= 0.0 ");
        return -1;
      }
      if (abs(vCM.px()/eCM) + abs(vCM.py()/eCM) + abs(vCM.pz()/eCM) > TINY) {
        cout<<"We are in a boosted frame. Boosting back"<<endl;
        for (unsigned int i=0; i<p.size(); ++i) p[i].bstback(vCM);
      }
    }

        
    //------------------------------------------------------------------------
    // QCD 2->n    
    if (nIn == 2 && isQCD) {
      
      // Massless only, setting all quark masses to zero
      double bMassSav   = coupl_mass_.bmass;
      coupl_mass_.bmass = 0.0;
      double cMassSav   = coupl_mass_.cmass;
      coupl_mass_.cmass = 0.0;
      double sMassSav   = coupl_mass_.smass;
      coupl_mass_.smass = 0.0;
      double dMassSav   = coupl_mass_.dmass;
      coupl_mass_.dmass = 0.0;
      double uMassSav   = coupl_mass_.umass;
      coupl_mass_.umass = 0.0;

      // Initialise helicity-dependence
      int nHels = 1;
      if (isPolarised) HEL[0] = 1;

      // ALL-GLUON AMPLITUDES ------ START
      
      // gg -> gluons 
      if (nQuarkTot == 0) {

        // Vincia is colour-ordered, MG starts with IS partons, then FS ones
        // Convention: pick icol so that FS partons are in ascending order
        map<int,int> mapVtoMG;
        // pos1Sav has to be 0
        if (pos1Sav != 0)
          cout<<" MG4interface : unrecognised colour order for gg -> gluons amplitude"<<endl;
        // How many partons inbetween the two IS gluo
        int nSep = pos2Sav-1;
        // First store IS gluon positions
        mapVtoMG[0] = pos1Sav;
        mapVtoMG[1] = pos2Sav;
        // Then store FS gluons, in ascending order
        int iMGnow=1;
        // Leave out first gluon (is initial)
        for (int i=1; i<(int)iGluon.size(); ++i) 
          if (iGluon[i] != pos2Sav) mapVtoMG[++iMGnow] = iGluon[i];
        //cout<<" gg -> gluons amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" nSep = "<<nSep
           // <<" nOut = "<<nOut<<endl;
        //for (int i=0; i<nMom; ++i) cout<<" iMG = "<<i<<" <- iV = "<<mapVtoMG[i]<<endl;

        // Set MG momenta and helicities, then evaluate MG matrix elements
        if (nOut == 2) {
          icol = 3; // F[1, 2, 3, 4]
          if (nSep == 1) icol = 4; // F[1, 3, 2, 4]
          else if (nSep == 2) icol = 6; // F[1, 3, 4, 2]
          double PNEW[4][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2gg_(&colorDepth,&icol,PNEW,HEL,&MGME2); 
        } else if (nOut == 3) {
          icol = 5; // F[1, 2, 3, 4, 5]
          if (nSep == 1) icol = 6; // F[1, 3, 2, 4, 5]
          else if (nSep == 2) icol = 18; // F[1, 3, 4, 2, 5]
          else if (nSep == 3) icol = 22; // F[1, 3, 4, 5, 2]
          double PNEW[5][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2ggg_(&colorDepth,&icol,PNEW,HEL,&MGME2); 
        } else if (nOut == 4) {
          icol = 9; // F[1, 2, 3, 4, 5, 6]
          if (nSep == 1) icol = 10; // F[1, 3, 2, 4, 5, 6]
          else if (nSep == 2) icol = 86; // F[1, 3, 4, 2, 5, 6]
          else if (nSep == 3) icol = 104; // F[1, 3, 4, 5, 2, 6]
          else if (nSep == 4) icol = 106; // F[1, 3, 4, 5, 6, 2]
          double PNEW[6][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2gggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }
      } // End gg -> gluons

      // ALL-GLUON AMPLITUDES ------ END

      // TWO-QUARK AMPLITUDES ------ START

      // qg -> q + gluons (& c.c.)
      else if (nQuarkIn == 1 && nQuarkOut == 1) {

        // Vincia is colour-ordered, MG starts with IS partons, then FS ones
        // Convention: pick icol so that FS partons are in ascending order
        map<int,int> mapVtoMG;
        // How many partons inbetween FS quark and IS gluon
        //int direction = 1;
        int nSep = pos1Sav-1;
        // Check if this is a qBar g -> qBar + gluons, to be charge-conjugated
        // If so, count separation from FS antiquark, now at the end of the chain
        if (pos2Sav == nMom -1) {
          // Standard colour order : qg -> q + gluons
          // First store quark positions
          mapVtoMG[0] = pos2Sav;
          mapVtoMG[2] = 0;
          // Store IS gluon
          mapVtoMG[1] = pos1Sav;
          // Then store FS gluons, in ascending order
          int iMG=2;
          for (int i=0; i<(int)iGluon.size(); ++i) 
            if (iGluon[i] != pos1Sav) mapVtoMG[++iMG] = iGluon[i];
        } else if (pos1Sav == 0) {
          // Reverse colour order : qbar g -> qbar + gluons
          //direction = -1;
          nSep = (nMom - 1) - (pos2Sav + 1);
          // First store quark positions
          mapVtoMG[0] = pos1Sav;
          mapVtoMG[2] = nMom - 1;
          // Store IS gluon
          mapVtoMG[1] = pos2Sav;
          // Then store FS gluons, in ascending order (reverse direction)
          int iMG=2;
          for (int i=iGluon.size()-1; i>=0; --i) 
            if (iGluon[i] != pos2Sav) mapVtoMG[++iMG] = iGluon[i];
        } else {
          cout<<" MG4interface : unrecognised colour order for qg -> q + gluons amplitude"<<endl;
        }
        //cout<<" tentative qg -> q + ng amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" idII = "<<id[pos1Sav]<<" "
            //<<id[pos2Sav]<<" nSep = "<<nSep<<" direction = "<<direction<<endl;
        //for (int i=0; i<nMom; ++i) cout<<" iMG = "<<i<<" <- iV = "<<mapVtoMG[i]<<endl;

        // Set MG momenta and helicities, then evaluate MG matrix elements
        if (nOut == 2) {
          icol = ((nSep == 0) ? 1 : 2); // T[ 3, 1, 2, 4] : T[ 3, 1, 4, 2]
          double PNEW[4][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sug2ug_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 3) {
          icol = 1; // T[ 3, 1, 2, 4, 5]
          if (nSep == 1) icol = 4; // T[ 3, 1, 4, 2, 5]
          else if (nSep == 2) icol = 5; // T[ 3, 1, 4, 5, 2]
          double PNEW[5][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sug2ugg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 4) {
          icol = 1; // T[ 3, 1, 2, 4, 5, 6]
          if (nSep == 1) icol = 13; // T[ 3, 1, 4, 2, 5, 6]
          else if (nSep == 2) icol = 17; // T[ 3, 1, 4, 5, 2, 6]
          else if (nSep == 3) icol = 18; // T[ 3, 1, 4, 5, 6, 2]
          double PNEW[6][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sug2uggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 5) {
          icol = 1; // T[ 3, 1, 2, 4, 5, 6, 7]
          if (nSep == 1) icol = 61; // T[ 3, 1, 4, 2, 5, 6, 7]
          else if (nSep == 2) icol = 81; // T[ 3, 1, 4, 5, 2, 6, 7]
          else if (nSep == 3) icol = 86; // T[ 3, 1, 4, 5, 6, 2, 7]
          else if (nSep == 4) icol = 87; // T[ 3, 1, 4, 5, 6, 7, 2]
          double PNEW[7][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sug2ugggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }

      } // End qg -> q + gluons
      
      // qqbar -> gluons
      else if (nQuarkIn == 2 && nQuarkOut == 0) {
        
        map<int,int> mapVtoMG;
        // pos1Sav has to be 0 (incoming qbar)
        // pos2Sav has to be nMom-1 (incoming q)
        if (pos1Sav != 0 || pos2Sav != nMom-1)
          cout<<" MG4interface : unrecognised colour order for qqbar -> gluons amplitude"<<endl;
        // First store IS quark positions
        mapVtoMG[0] = pos2Sav;
        mapVtoMG[1] = pos1Sav;
        // Then store FS gluons, in ascending order
        int iMGnow=1;
        for (int i=0; i<(int)iGluon.size(); ++i) 
          mapVtoMG[++iMGnow] = iGluon[i];
        //cout<<" qqbar -> gluons amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" nOut = "<<nOut<<endl;
        //for (int i=0; i<nMom; ++i) cout<<" iMG = "<<i<<" <- iV = "<<mapVtoMG[i]<<endl;

        // Set MG momenta and helicities, then evaluate MG matrix elements
        if (nOut == 2) {
          icol = 1; // T[ 2, 1, 3, 4]
          double PNEW[4][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          suux2gg_(&colorDepth,&icol,PNEW,HEL,&MGME2); 
        } else if (nOut == 3) {
          icol = 1; // T[ 2, 1, 3, 4, 5]
          double PNEW[5][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          suux2ggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 4) {
          icol = 1; //  T[ 2, 1, 3, 4, 5, 6]
          double PNEW[6][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          suux2gggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 5) {
          icol = 1; // T[ 2, 1, 3, 4, 5, 6, 7]
          double PNEW[7][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          suux2ggggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }
        
      } // End qqbar -> gluons

      // gg -> qqbar + gluons
      else if (nQuarkIn == 0 && nQuarkOut == 2) {
        
        map<int,int> mapVtoMG;
        // First store IS gluon positions
        mapVtoMG[0] = pos1Sav;
        mapVtoMG[1] = pos2Sav;
        // Then store FS quark positions
        mapVtoMG[2] = 0;
        mapVtoMG[3] = nMom-1;
        // Then store FS gluons, in ascending order
        int iMGnow=3;
        for (int i=0; i<(int)iGluon.size(); ++i) 
          if ((iGluon[i] != pos1Sav) && (iGluon[i] != pos2Sav)) 
            mapVtoMG[++iMGnow] = iGluon[i];
        //cout<<" gg -> qqbar + gluons amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" nOut = "<<nOut<<endl;
        //for (int i=0; i<nMom; ++i) cout<<" iMG = "<<i<<" <- iV = "<<mapVtoMG[i]<<endl;

        // Set MG momenta and helicities, then evaluate MG matrix elements
        if (nOut == 2) {
          if (pos1Sav == 1 && pos2Sav == 2)
            icol = 2; // T[ 3, 4, 1, 2] 
          else cout<<" MG4interface : unrecognised colour order for gg -> qqbar + gluons amplitude"<<endl;
          double PNEW[4][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2uux_(&colorDepth,&icol,PNEW,HEL,&MGME2); 
        } else if (nOut == 3) {
          if (pos1Sav == 1 && pos2Sav == 2)
            icol = 6; // T[ 3, 4, 1, 2, 5]
          else if (pos1Sav == 1 && pos2Sav == 3)
            icol = 4; // T[ 3, 4, 1, 5, 2]
          else if (pos1Sav == 2 && pos2Sav == 3)
            icol = 5; // T[ 3, 4, 5, 1, 2]
          else cout<<" MG4interface : unrecognised colour order for gg -> qqbar + gluons amplitude"<<endl;
          double PNEW[5][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2uuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 4) {
          if (pos1Sav == 1 && pos2Sav == 2)
            icol = 24; // T[ 3, 4, 1, 2, 5, 6]
          else if (pos1Sav == 1 && pos2Sav == 3)
            icol = 17; // T[ 3, 4, 1, 5, 2, 6]
          else if (pos1Sav == 1 && pos2Sav == 4)
            icol = 19; // T[ 3, 4, 1, 5, 6, 2]
          else if (pos1Sav == 2 && pos2Sav == 3)
            icol = 18; // T[ 3, 4, 5, 1, 2, 6]
          else if (pos1Sav == 2 && pos2Sav == 4)
            icol = 13; // T[ 3, 4, 5, 1, 6, 2]
          else if (pos1Sav == 3 && pos2Sav == 4)
            icol = 15; // T[ 3, 4, 5, 6, 1, 2]
          else cout<<" MG4interface : unrecognised colour order for gg -> qqbar + gluons amplitude"<<endl;
          double PNEW[6][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        } else if (nOut == 5) {
          if (pos1Sav == 1 && pos2Sav == 2)
            icol = 119; // T[ 3, 4, 1, 2, 5, 6, 7]
          else if (pos1Sav == 1 && pos2Sav == 3)
            icol =  93; // T[ 3, 4, 1, 5, 2, 6, 7]
          else if (pos1Sav == 1 && pos2Sav == 4)
            icol =  90; // T[ 3, 4, 1, 5, 6, 2, 7]
          else if (pos1Sav == 1 && pos2Sav == 5)
            icol =  91; // T[ 3, 4, 1, 5, 6, 7, 2]
          else if (pos1Sav == 2 && pos2Sav == 3)
            icol =  94; // T[ 3, 4, 5, 1, 2, 6, 7]
          else if (pos1Sav == 2 && pos2Sav == 4)
            icol =  69; // T[ 3, 4, 5, 1, 6, 2, 7]
          else if (pos1Sav == 2 && pos2Sav == 5)
            icol =  70; // T[ 3, 4, 5, 1, 6, 7, 2]
          else if (pos1Sav == 3 && pos2Sav == 4)
            icol =  73; // T[ 3, 4, 5, 6, 1, 2, 7]
          else if (pos1Sav == 3 && pos2Sav == 5)
            icol =  65; // T[ 3, 4, 5, 6, 1, 7, 2]
          else if (pos1Sav == 4 && pos2Sav == 5)
            icol =  66; // T[ 3, 4, 5, 6, 7, 1, 2]
          else cout<<" MG4interface : unrecognised colour order for gg -> qqbar + gluons amplitude"<<endl;
          double PNEW[7][4];
          for (int iMG=nMom-1; iMG>=0; --iMG) {
            Vec4toMG(p[mapVtoMG[iMG]],PNEW[iMG]);
            if (HEL[0] > 0) HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
            nHels *= 2;
          }
          //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
          sgg2uuxggg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }
        
      } // End gg -> qqbar + gluons

      // TWO-QUARK AMPLITUDES ------ END

      // FOUR-QUARK AMPLITUDES ------ START
      
      else if (nQuarkTot == 4) {

        // Check if all quarks have same flavour
        // Compare to first quark
        int idComp = abs(id[0]);
        int count  = 0;
        for (int i=1; i<nMom; i++) if (abs(id[i]) == idComp) count++;
        bool sameFlav = (count > 1);
        
        if (nOut == 2) { // (q, qbar, q, qbar)
          
          double PNEW[4][4];
          int id1 = id[pos1Sav];
          int id2 = id[pos2Sav];
          
          //cout<<" 4-quark amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" idII = "
              //<<id1<<" "<<id2<<endl;
          //for (int i=0; i<nMom; i++) cout<<id[i]<<" "; cout<<endl;
          
          // qq -> qq and qbarqbar -> qbarqbar
          if (id1 == id2) {
            if (!sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, qbar)
              Vec4toMGQCD(p, 0, 2, 1, 3, PNEW);
              icol = 1; // T[ 4, 2]T[ 3, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {0, 2, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uu_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar, q, -qbar)
              Vec4toMGQCD(p, 3, 1, 0, 2, PNEW);
              icol = 2; // T[ 4, 1]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {3, 1, 0, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uu_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbar and qqbar -> q'qbar'
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 3, PNEW);
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {1, 0, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar, -q, -qbar)
              Vec4toMGQCD(p, 3, 2, 0, 1, PNEW);
              icol = 1; // T[ 2, 1]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {3, 2, 0, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, qbar, q, -qbar)
              Vec4toMGQCD(p, 3, 0, 2, 1, PNEW);
              icol = (sameFlav ? 2 : 1); // T[ 3, 1]T[ 2, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {3, 0, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -qbar, -q, qbar)
              Vec4toMGQCD(p, 1, 2, 0, 3, PNEW);
              icol = (sameFlav ? 2 : 1); // T[ 3, 1]T[ 2, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {1, 2, 0, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qq' -> qq' and qbarqbar' -> qbarqbar'
          else if (id1*id2 > 0) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, qbar)
              Vec4toMGQCD(p, 0, 2, 3, 1, PNEW);
              icol = 1; // T[ 4, 1]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {0, 2, 3, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2ud_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar, q, -qbar)
              Vec4toMGQCD(p, 1, 3, 2, 0, PNEW);
              icol = 1; // T[ 4, 1]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {1, 3, 2, 0};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2ud_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar' and qbarq' -> qbarq'
          else if (id1*id2 < 0) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 3, PNEW);
              icol = 1; // T[ 2, 1]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {1, 0, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar, -q, -qbar)
              Vec4toMGQCD(p, 3, 2, 0, 1, PNEW);
              icol = 1; // T[ 2, 1]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[4] = {3, 2, 0, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          
        }
        
        else if (nOut == 3) { // (q, qbar, q, g, qbar)
          
          double PNEW[5][4];
          int id1 = id[pos1Sav];
          int id2 = id[pos2Sav];
          
          //cout<<" 4-quark + g amplitude with II = "<<pos1Sav<<" "<<pos2Sav<<" idII = "
              //<<id1<<" "<<id2<<endl;
          //for (int i=0; i<nMom; i++) cout<<id[i]<<" "; cout<<endl;
          
          // qq -> qqg and qbarqbar -> qbarqbarg
          if (id1 == id2) {
            if (!sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, g, qbar)
              Vec4toMGQCD(p, 0, 2, 1, 4, 3, PNEW);
              icol = 3; // T[ 4, 2, 5]T[ 3, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {0, 2, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uug_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -qbar, q, g, -qbar)
              Vec4toMGQCD(p, 4, 1, 0, 2, 3, PNEW);
              icol = 2; // T[ 4, 1, 5]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {4, 1, 0, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uug_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbarg and qqbar -> q'qbar'g
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, g, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 4, 3, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {1, 0, 2, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, qbar, -q, g, -qbar)
              Vec4toMGQCD(p, 4, 2, 0, 1, 3, PNEW);
              icol = 1; // T[ 2, 1, 5]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {4, 2, 0, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q, qbar, q, g, -qbar)
              Vec4toMGQCD(p, 4, 0, 2, 1, 3, PNEW);
              icol = (sameFlav ? 3 : 1); // T[ 3, 1, 5]T[ 2, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {4, 0, 2, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -qbar, -q, g, qbar)
              Vec4toMGQCD(p, 1, 2, 0, 4, 3, PNEW);
              icol = (sameFlav ? 4 : 2); // T[ 3, 1]T[ 2, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {1, 2, 0, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qq' -> qq'g and qbarqbar' -> qbarqbar'g
          else if (id1*id2 > 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, g, qbar)
              Vec4toMGQCD(p, 0, 2, 4, 1, 3, PNEW);
              icol = 2; // T[ 4, 1]T[ 3, 2, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {0, 2, 4, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -qbar, q, g, -qbar)
              Vec4toMGQCD(p, 1, 4, 2, 0, 3, PNEW);
              icol = 2; // T[ 4, 1]T[ 3, 2, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {1, 4, 2, 0, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar'g and qbarq' -> qbarq'g
          else if (id1*id2 < 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, g, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 4, 3, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {1, 0, 2, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, qbar, -q, g, -qbar)
              Vec4toMGQCD(p, 4, 2, 0, 1, 3, PNEW);
              icol = 1; // T[ 2, 1, 5]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {4, 2, 0, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qg -> qqqbar and qg -> qq'qbar'
          // qbarg -> qbarqbarq and qbarg -> qbarqbar'q'
          else if (id1 == 21 || id2 == 21) {
            if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, qbar, q, -g, qbar)
              Vec4toMGQCD(p, 0, 3, 4, 1, 2, PNEW);
              icol = (sameFlav ? 3 : 1); // T[ 4, 1]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {0, 3, 4, 1, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar, q, -g, qbar)
              Vec4toMGQCD(p, 1, 3, 2, 0, 4, PNEW);
              icol = (sameFlav ? 3 : 1); // T[ 4, 1]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {1, 3, 2, 0, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar, -q, -g, qbar)
              Vec4toMGQCD(p, 2, 3, 1, 4, 0, PNEW);
              icol = (sameFlav ? 4 : 2); // T[ 4, 1, 2]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {2, 3, 1, 4, 0};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q, qbar, q, -g, -qbar)
              Vec4toMGQCD(p, 4, 3, 0, 2, 1, PNEW);
              icol = (sameFlav ? 4 : 2); // T[ 4, 1, 2]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[5] = {4, 3, 0, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          
        }
        
        else if (nOut == 4 && id[3] == 21 && id[4] == 21) { // (q, qbar, q, g, g, qbar)
          
          double PNEW[6][4];
          int id1 = id[pos1Sav];
          int id2 = id[pos2Sav];
          
          //cout<<" 4-quark + gg amplitude (q, qbar, q, g, g, qbar) with II = "
              //<<pos1Sav<<" "<<pos2Sav<<" idII = "<<id1<<" "<<id2<<endl;
          //for (int i=0; i<nMom; i++) cout<<id[i]<<" "; cout<<endl;
          
          // qq -> qqgg and qbarqbar -> qbarqbargg
          if (id1 == id2 && id1 != 21) {
            if (!sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, g, g, qbar)
              Vec4toMGQCD(p, 0, 2, 1, 5, 3, 4, PNEW);
              icol = 5; // T[ 4, 2, 5, 6]T[ 3, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 2, 1, 5, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uugg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 5) {
              // (q, -qbar, q, g, g, -qbar)
              Vec4toMGQCD(p, 5, 1, 0, 2, 3, 4, PNEW);
              icol = 7; // T[ 4, 1, 5, 6]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 1, 0, 2, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uugg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbargg and qqbar -> q'qbar'gg
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, g, g, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 5, 3, 4, PNEW);
              icol = 6; // T[ 2, 1]T[ 3, 4, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 0, 2, 5, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 0 && pos2Sav == 5) {
              // (-q, qbar, q, g, g, -qbar)
              Vec4toMGQCD(p, 5, 0, 2, 1, 3, 4, PNEW);
              icol = (sameFlav ? 7 : 1); // T[ 3, 1, 5, 6]T[ 2, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 0, 2, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -qbar, -q, g, g, qbar)
              Vec4toMGQCD(p, 1, 2, 0, 5, 3, 4, PNEW);
              icol = (sameFlav ? 12 : 6); // T[ 3, 1]T[ 2, 4, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 2, 0, 5, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, qbar, -q, g, g, -qbar)
              Vec4toMGQCD(p, 5, 2, 0, 1, 3, 4, PNEW);
              icol = 1; // T[ 2, 1, 5, 6]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qq' -> qq'gg and qbarqbar' -> qbarqbar'gg
          else if (id1*id2 > 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, g, g, qbar)
              Vec4toMGQCD(p, 0, 2, 5, 1, 3, 4, PNEW);
              icol = 5; // T[ 4, 1]T[ 3, 2, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 2, 5, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 5) {
              // (q, -qbar, q, g, g, -qbar)
              Vec4toMGQCD(p, 1, 5, 2, 0, 3, 4, PNEW);
              icol = 5; // T[ 4, 1]T[ 3, 2, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 5, 2, 0, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar'gg and qbarq' -> qbarq'gg
          else if (id1*id2 < 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, g, g, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 5, 3, 4, PNEW);
              icol = 6; // T[ 2, 1]T[ 3, 4, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 0, 2, 5, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, qbar, -q, g, g, -qbar)
              Vec4toMGQCD(p, 5, 2, 0, 1, 3, 4, PNEW);
              icol = 1; // T[ 2, 1, 5, 6]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qg -> qqqbarg and qg -> qq'qbar'g
          // qbarg -> qbarqbarqg and qbarg -> qbarqbar'q'g
          else if ( (id1 == 21 && id2 != 21 ) || (id2 == 21 && id1 != 21) ) {
            if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, qbar, q, -g, g, qbar)
              Vec4toMGQCD(p, 0, 3, 5, 1, 2, 4, PNEW);
              icol = (sameFlav ? 9 : 3); // T[ 4, 1]T[ 3, 5, 6, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 3, 5, 1, 2, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q, qbar, q, g, -g, qbar)
              Vec4toMGQCD(p, 0, 4, 5, 1, 2, 3, PNEW);
              icol = (sameFlav ? 8 : 2); // T[ 4, 1]T[ 3, 5, 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 4, 5, 1, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar, q, -g, g, qbar)
              Vec4toMGQCD(p, 1, 3, 2, 0, 5, 4, PNEW);
              icol = (sameFlav ? 8 : 2); // T[ 4, 1]T[ 3, 5, 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 3, 2, 0, 5, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -qbar, q, g, -g, qbar)
              Vec4toMGQCD(p, 1, 4, 2, 0, 5, 3, PNEW);
              icol = (sameFlav ? 9 : 3); // T[ 4, 1]T[ 3, 5, 6, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 4, 2, 0, 5, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar, -q, -g, g, qbar)
              Vec4toMGQCD(p, 2, 3, 1, 5, 0, 4, PNEW);
              icol = (sameFlav ? 11 : 5); // T[ 4, 1, 6, 2]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 3, 1, 5, 0, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, qbar, -q, g, -g, qbar)
              Vec4toMGQCD(p, 2, 4, 1, 5, 0, 3, PNEW);
              icol = (sameFlav ? 10 : 4); // T[ 4, 1, 2, 6]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 4, 1, 5, 0, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q, qbar, q, -g, g, -qbar)
              Vec4toMGQCD(p, 5, 3, 0, 2, 1, 4, PNEW);
              icol = (sameFlav ? 10 : 4); // T[ 4, 1, 2, 6]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 3, 0, 2, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 4 && pos2Sav == 5) {
              // (q, qbar, q, g, -g, -qbar)
              Vec4toMGQCD(p, 5, 4, 0, 2, 1, 3, PNEW);
              icol = (sameFlav ? 11 : 5); // T[ 4, 1, 6, 2]T[ 3, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 4, 0, 2, 1, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // gg -> qqbarqqbar and gg -> qqbarq'qbar'
          else if (id1 == 21 && id2 == 21) {
            if (pos1Sav == 3 && pos2Sav == 4) {
              // (q, qbar, q, -g, -g, qbar)
              Vec4toMGQCD(p, 3, 4, 0, 5, 2, 1, PNEW);
              icol = (sameFlav ? 5 : 5); // T[ 5, 4, 1, 2]T[ 3, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 4, 0, 5, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sgg2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sgg2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          
        }
        
        else if (nOut == 4 && id[1] == 21 && id[4] == 21) { // (q, g, qbar, q, g, qbar)
          
          double PNEW[6][4];
          int id1 = id[pos1Sav];
          int id2 = id[pos2Sav];
          
          //cout<<" 4-quark + gg amplitude (q, g, qbar, q, g, qbar) with II = "
              //<<pos1Sav<<" "<<pos2Sav<<" idII = "<<id1<<" "<<id2<<endl;
          //for (int i=0; i<nMom; i++) cout<<id[i]<<" "; cout<<endl;
          
          // qq -> qqgg and qbarqbar -> qbarqbargg
          if (id1 == id2 && id1 != 21) {
            if (!sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, g, qbar, -q, g, qbar)
              Vec4toMGQCD(p, 0, 3, 2, 5, 1, 4, PNEW);
              icol = 2; // T[ 4, 2, 6]T[ 3, 1, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 3, 2, 5, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uugg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, g, -qbar, q, g, -qbar)
              Vec4toMGQCD(p, 5, 2, 0, 3, 1, 4, PNEW);
              icol = 10; // T[ 4, 1, 6]T[ 3, 2, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 3, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uugg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbargg and qqbar -> q'qbar'gg
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, g, -qbar, q, g, qbar)
              Vec4toMGQCD(p, 2, 0, 3, 5, 4, 1, PNEW);
              icol = 4; // T[ 2, 1, 6]T[ 3, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 0, 3, 5, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q, g, qbar, -q, g, -qbar)
              Vec4toMGQCD(p, 5, 3, 0, 2, 4, 1, PNEW);
              icol = 3; // T[ 2, 1, 5]T[ 3, 4, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 3, 0, 2, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            }
            else if (pos1Sav == 0 && pos2Sav == 5) {
              // (-q, g, qbar, q, g, -qbar)
              Vec4toMGQCD(p, 5, 0, 3, 2, 4, 1, PNEW);
              icol = (sameFlav ? 8 : 2); // T[ 3, 1, 5]T[ 2, 4, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 0, 3, 2, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, g, -qbar, -q, g, qbar)
              Vec4toMGQCD(p, 2, 3, 0, 5, 4, 1, PNEW);
              icol = (sameFlav ? 10 : 4); // T[ 3, 1, 6]T[ 2, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 3, 0, 5, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) suux2uuxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          suux2ddxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qq' -> qq'gg and qbarqbar' -> qbarqbar'gg
          else if (id1*id2 > 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, g, qbar, -q, g, qbar)
              Vec4toMGQCD(p, 0, 3, 5, 2, 1, 4, PNEW);
              icol = 2; // T[ 4, 1, 5]T[ 3, 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 3, 5, 2, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, g, -qbar, q, g, -qbar)
              Vec4toMGQCD(p, 2, 5, 3, 0, 1, 4, PNEW);
              icol = 2; // T[ 4, 1, 5]T[ 3, 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 5, 3, 0, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar'gg and qbarq' -> qbarq'gg
          else if (id1*id2 < 0 && id1 != 21 && id2 != 21) {
            if (sameFlav) cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, g, -qbar, q, g, qbar)
              Vec4toMGQCD(p, 2, 0, 3, 5, 1, 4, PNEW);
              icol = 3; // T[ 2, 1, 5]T[ 3, 4, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 0, 3, 5, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q, g, qbar, -q, g, -qbar)
              Vec4toMGQCD(p, 5, 3, 0, 2, 1, 4, PNEW);
              icol = 4; // T[ 2, 1, 6]T[ 3, 4, 5]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 3, 0, 2, 1, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxgg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qg -> qqqbarg and qg -> qq'qbar'g
          // qbarg -> qbarqbarqg and qbarg -> qbarqbar'q'g
          else if ( (id1 == 21 && id2 != 21 ) || (id2 == 21 && id1 != 21) ) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -g, qbar, q, g, qbar)
              Vec4toMGQCD(p, 0, 1, 5, 2, 3, 4, PNEW);
              icol = (sameFlav ? 12 : 6); // T[ 4, 1, 2]T[ 3, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 1, 5, 2, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q, g, qbar, q, -g, qbar)
              Vec4toMGQCD(p, 0, 4, 5, 2, 3, 1, PNEW);
              icol = (sameFlav ? 7 : 1); // T[ 4, 1, 6]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 4, 5, 2, 3, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -g, -qbar, q, g, qbar)
              Vec4toMGQCD(p, 2, 1, 3, 0, 5, 4, PNEW);
              icol = (sameFlav ? 12 : 6); // T[ 4, 1, 2]T[ 3, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 1, 3, 0, 5, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -g, qbar, -q, g, qbar)
              Vec4toMGQCD(p, 3, 1, 2, 5, 0, 4, PNEW);
              icol = (sameFlav ? 7 : 1); // T[ 4, 1, 6]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 1, 2, 5, 0, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 5) {
              // (q, -g, qbar, q, g, -qbar)
              Vec4toMGQCD(p, 5, 1, 0, 3, 2, 4, PNEW);
              icol = (sameFlav ? 7 : 1); // T[ 4, 1, 6]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 1, 0, 3, 2, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, g, -qbar, q, -g, qbar)
              Vec4toMGQCD(p, 2, 4, 3, 0, 5, 1, PNEW);
              icol = (sameFlav ? 7 : 1); // T[ 4, 1, 6]T[ 3, 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 4, 3, 0, 5, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q, g, qbar, -q, -g, qbar)
              Vec4toMGQCD(p, 3, 4, 2, 5, 0, 1, PNEW);
              icol = (sameFlav ? 12 : 6); // T[ 4, 1, 2]T[ 3, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 4, 2, 5, 0, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 4 && pos2Sav == 5) {
              // (q, g, qbar, q, -g, -qbar)
              Vec4toMGQCD(p, 5, 4, 0, 3, 2, 1, PNEW);
              icol = (sameFlav ? 12 : 6); // T[ 4, 1, 2]T[ 3, 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 4, 0, 3, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sug2uuuxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sug2uddxg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // gg -> qqbarqqbar and gg -> qqbarq'qbar'
          else if (id1 == 21 && id2 == 21) {
            if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -g, qbar, q, -g, qbar)
              Vec4toMGQCD(p, 1, 4, 0, 5, 3, 2, PNEW);
              icol = (sameFlav ? 4 : 4); // T[ 5, 4, 2]T[ 3, 6, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 4, 0, 5, 3, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              if (sameFlav) sgg2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
              else          sgg2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
            
          else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          
        }
        
        else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
        
      }

      // FOUR-QUARK AMPLITUDES ------ END
      
      // SIX-QUARK AMPLITUDES ------ START
      
      else if (nQuarkTot == 6 && nOut == 4) {
        
        // Check how many different flavours we have
        map<int,int> flavours;
        for (int i=1; i<=5; ++i) flavours[i] = 0;
        for (int i=0; i<nMom; ++i) flavours[abs(id[i])]++;
        int countSameFlavour = 0; // 0 if 0 or 1 quark pair, 1 if 2 quark pairs, 2 if 3 quark pais
        for (int i=1; i<=5; ++i) countSameFlavour = max( max((flavours[i]-2)/2,0) , countSameFlavour);
          
        double PNEW[6][4];
        int id1 = id[pos1Sav];
        int id2 = id[pos2Sav];
	//int pos1Old = pos1Sav;
	//int pos2Old = pos2Sav;
        
        if (countSameFlavour == 2) {
          
          //cout<<" 6-quark amplitude (same flavour) with II = "<<pos1Old<<" "<<pos2Old<<" idII = "
              //<<id[pos1Old]<<" "<<id[pos2Old]<<endl;
          
          // qq -> qqqqbar and qbarqbar -> qbarqbarqqbar
          if (id1 == id2) {
            if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar, -q, qbar, q, qbar)
              Vec4toMGQCD(p, 0, 2, 1, 3, 5, 4, PNEW);
              icol = 4; // T[ 3, 1]T[ 4, 2]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 2, 1, 3, 5, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q, qbar, q, qbar, -q, qbar)
              Vec4toMGQCD(p, 0, 4, 1, 3, 5, 2, PNEW);
              icol = 2; // T[ 3, 1]T[ 4, 6]T[ 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 4, 1, 3, 5, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, qbar, -q, qbar, -q, qbar)
              Vec4toMGQCD(p, 2, 4, 1, 3, 5, 0, PNEW);
              icol = 3; // T[ 3, 6]T[ 4, 1]T[ 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 4, 1, 3, 5, 0};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar, q, -qbar, q, qbar)
              Vec4toMGQCD(p, 1, 3, 0, 2, 4, 5, PNEW);
              icol = 4; // T[ 3, 1]T[ 4, 2]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 3, 0, 2, 4, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 5) {
              // (q, -qbar, q, qbar, q, -qbar)
              Vec4toMGQCD(p, 1, 5, 0, 2, 4, 3, PNEW);
              icol = 2; // T[ 3, 1]T[ 4, 6]T[ 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 5, 0, 2, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q, qbar, q, -qbar, q, -qbar)
              Vec4toMGQCD(p, 3, 5, 0, 2, 4, 1, PNEW);
              icol = 3; // T[ 3, 6]T[ 4, 1]T[ 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 5, 0, 2, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 4-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbarqqbar
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar, q, -qbar, q, qbar)
              Vec4toMGQCD(p, 1, 0, 2, 3, 4, 5, PNEW);
              icol = 4; // T[ 2, 1]T[ 3, 4]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 0, 2, 3, 4, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, qbar, q, -qbar, q, qbar)
              Vec4toMGQCD(p, 3, 0, 2, 1, 4, 5, PNEW);
              icol = 2; // T[ 2, 4]T[ 3, 1]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 0, 2, 1, 4, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 5) {
              // (-q, qbar, q, qbar, q, -qbar)
              Vec4toMGQCD(p, 5, 0, 2, 1, 4, 3, PNEW);
              icol = 5; // T[ 2, 4]T[ 3, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 0, 2, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -qbar, -q, qbar, q, qbar)
              Vec4toMGQCD(p, 1, 2, 0, 3, 4, 5, PNEW);
              icol = 2; // T[ 2, 4]T[ 3, 1]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 2, 0, 3, 4, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -qbar, q, qbar, -q, qbar)
              Vec4toMGQCD(p, 1, 4, 0, 3, 2, 5, PNEW);
              icol = 3; // T[ 5, 4]T[ 3, 1]T[ 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 4, 0, 3, 2, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar, -q, -qbar, q, qbar)
              Vec4toMGQCD(p, 3, 2, 0, 1, 4, 5, PNEW);
              icol = 4; // T[ 2, 1]T[ 3, 4]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 2, 0, 1, 4, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, qbar, -q, qbar, q, -qbar)
              Vec4toMGQCD(p, 5, 2, 0, 1, 4, 3, PNEW);
              icol = 6; // T[ 5, 1]T[ 3, 4]T[ 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q, qbar, q, -qbar, -q, qbar)
              Vec4toMGQCD(p, 3, 4, 0, 1, 2, 5, PNEW);
              icol = 6; // T[ 5, 1]T[ 3, 4]T[ 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 4, 0, 1, 2, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 4 && pos2Sav == 5) {
              // (q, qbar, q, qbar, -q, -qbar)
              Vec4toMGQCD(p, 5, 4, 0, 1, 2, 3, PNEW);
              icol = 4; // T[ 2, 1]T[ 3, 4]T[ 5, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 4, 0, 1, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
            
        }
        
        else if (countSameFlavour == 1) {
          
          // Order as (q', qbar, q, qbar, q, qbar')
          int absFlav = 0;
          for (int i=1; i<=5; ++i) if (flavours[i] == 2) absFlav = i;
          pair<int,int> flavourPos; // quark first, antiquark second
          for (int i=0; i<nMom; ++i) { 
            if (abs(id[i]) == absFlav) {
              if (i % 2 == 0) // quarks sitting on even numbers
                flavourPos.first = i;
              else 
                flavourPos.second = i;
            }
          }
          vector<int> order = orderQQBqqbqqbQCD(flavourPos);
          vector<Vec4> pTmp = p;
          for (int i=0; i<nMom; ++i) p[i] = pTmp[order[i]];
          // Update position of incoming quarks
          for (int i=0; i<nMom; ++i) if (order[i] == pos1Sav) {
            pos1Sav = i;
            break;
          }
          for (int i=0; i<nMom; ++i) if (order[i] == pos2Sav) {
            pos2Sav = i;
            break;
          }
          if (pos2Sav < pos1Sav) {
            int posTmp = pos1Sav;
            pos1Sav = pos2Sav;
            pos2Sav = posTmp;
          }
          //cout<<" 6-quark amplitude (2 diff flavour) with II = "<<pos1Old<<" "<<pos2Old<<" idII = "
              //<<id[pos1Old]<<" "<<id[pos2Old]<<endl
              //<<"                                         II = "<<pos1Sav<<" "<<pos2Sav<<endl;
          //for (int i=0; i<nMom; ++i) cout<<" "<<i<<" -> "<<order[i]<<endl;
          
          // qq -> qqq'qbar' and qbarqbar -> qbarqbarq'qbar'
          if (id1 == id2) {
            if (pos1Sav == 1 && pos2Sav == 3) {
              // (q', -qbar, q, -qbar, q, qbar')
              Vec4toMGQCD(p, 1, 3, 2, 4, 0, 5, PNEW);
              icol = 3; // T[ 5, 11T[ 3, 2]T[ 4, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 3, 2, 4, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q', qbar, -q, qbar, -q, qbar')
              Vec4toMGQCD(p, 2, 4, 1, 3, 5, 0, PNEW);
              icol = 4; // T[ 3, 6]T[ 4, 1]T[ 5, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 4, 1, 3, 5, 0};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suu2uuddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          // qqbar -> qqbarq'qbar' and q'qbar' -> qqbarqqbar
          else if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 5) {
              // (-q', qbar, q, qbar, q, -qbar')
              Vec4toMGQCD(p, 5, 0, 2, 1, 4, 3, PNEW);
              icol = 3; // T[ 2, 4]T[ 3, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 0, 2, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2ddxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q', -qbar, -q, qbar, q, qbar')
              Vec4toMGQCD(p, 1, 2, 4, 3, 0, 5, PNEW);
              icol = 3; // T[ 2, 4]T[ 3, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 2, 4, 3, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q', -qbar, q, qbar, -q, qbar')
              Vec4toMGQCD(p, 1, 4, 2, 3, 0, 5, PNEW);
              icol = 1; // T[ 3, 4]T[ 2, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 4, 2, 3, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q', qbar, -q, -qbar, q, qbar')
              Vec4toMGQCD(p, 3, 2, 4, 1, 0, 5, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 2, 4, 1, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q', qbar, q, -qbar, -q, qbar')
              Vec4toMGQCD(p, 3, 4, 2, 1, 0, 5, PNEW);
              icol = 4; // T[ 3, 1]T[ 2, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 4, 2, 1, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2uuxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          // q'q -> q'qqqbar and qbar'qbar -> qbar'qbarqqbar
          else if (id1*id2 > 0) {
            if (pos1Sav == 1 && pos2Sav == 5) {
              // (q', -qbar, q, qbar, q, -qbar')
              Vec4toMGQCD(p, 5, 1, 0, 2, 4, 3, PNEW);
              icol = 3; // T[ 4, 6]T[ 3, 2]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 1, 0, 2, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q', qbar, q, -qbar, q, -qbar')
              Vec4toMGQCD(p, 5, 3, 0, 2, 4, 1, PNEW);
              icol = 2; // T[ 4, 2]T[ 3, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 3, 0, 2, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q', qbar, -q, qbar, q, qbar')
              Vec4toMGQCD(p, 0, 2, 5, 1, 3, 4, PNEW);
              icol = 4; // T[ 4, 1]T[ 5, 2]T[ 3, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 2, 5, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q', qbar, q, qbar, -q, qbar')
              Vec4toMGQCD(p, 0, 4, 5, 1, 3, 2, PNEW);
              icol = 1; // T[ 4, 1]T[ 5, 6]T[ 3, 2]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 4, 5, 1, 3, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar'qqbar and q'qbar -> q'qbarqqbar
          else if (id1*id2 < 0) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q', -qbar, q, qbar, q, qbar')
              Vec4toMGQCD(p, 1, 0, 2, 5, 4, 3, PNEW);
              icol = 4; // T[ 2, 1]T[ 3, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 0, 2, 5, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q', qbar, q, -qbar, q, qbar')
              Vec4toMGQCD(p, 3, 0, 2, 5, 4, 1, PNEW);
              icol = 1; // T[ 2, 6]T[ 3, 1]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 0, 2, 5, 4, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxuux_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q', qbar, -q, qbar, q, -qbar')
              Vec4toMGQCD(p, 5, 2, 0, 1, 4, 3, PNEW);
              icol = 3; // T[ 2, 6]T[ 5, 1]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 4 && pos2Sav == 5) {
              // (q', qbar, q, qbar, -q, -qbar')
              Vec4toMGQCD(p, 5, 4, 0, 1, 2, 3, PNEW);
              icol = 2; // T[ 5, 6]T[ 2, 1]T[ 3, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 4, 0, 1, 2, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          
        }
        
        else if (countSameFlavour == 0) {
          
          // Order as (q, qbar', q', qbar'', q'', qbar)
          int absFlav = abs(id1); // pick random flavour
          pair<int,int> flavourPos; // quark first, antiquark second
          for (int i=0; i<nMom; ++i) { 
            if (abs(id[i]) == absFlav) {
              if (i % 2 == 0) // quarks sitting on even numbers
                flavourPos.first = i;
              else 
                flavourPos.second = i;
            }
          }
          vector<int> order = orderQQBqqbqqbQCD(flavourPos);
          vector<Vec4> pTmp = p;
          for (int i=0; i<nMom; ++i) p[i] = pTmp[order[i]];
          // Update position of incoming quarks
          for (int i=0; i<nMom; ++i) if (order[i] == pos1Sav) {
            pos1Sav = i;
            break;
          }
          for (int i=0; i<nMom; ++i) if (order[i] == pos2Sav) {
            pos2Sav = i;
            break;
          }
          if (pos2Sav < pos1Sav) {
            int posTmp = pos1Sav;
            pos1Sav = pos2Sav;
            pos2Sav = posTmp;
          }
          //cout<<" 6-quark amplitude (3 diff flavour) with II = "<<pos1Old<<" "<<pos2Old<<" idII = "
              //<<id[pos1Old]<<" "<<id[pos2Old]<<endl;
              //<<"                                         II = "<<pos1Sav<<" "<<pos2Sav<<endl;
          //for (int i=0; i<nMom; ++i) cout<<" "<<i<<" -> "<<order[i]<<endl;
          
          // qqbar -> q'qbar'q''qbar''
          if (id1 == -id2) {
            if (pos1Sav == 0 && pos2Sav == 5) {
              // (-q, qbar', q', qbar'', q'', -qbar)
              Vec4toMGQCD(p, 5, 0, 2, 1, 4, 3, PNEW);
              icol = 1; // T[ 5, 1]T[ 2, 4]T[ 3, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 0, 2, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2ddxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // (q, -qbar', -q', qbar'', q'', qbar)
              Vec4toMGQCD(p, 1, 2, 0, 5, 4, 3, PNEW);
              icol = 2; // T[ 3, 1]T[ 5, 4]T[ 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 2, 0, 5, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2ddxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q, qbar', q', -qbar'', -q'', qbar)
              Vec4toMGQCD(p, 3, 4, 2, 1, 0, 5, PNEW);
              icol = 2; // T[ 3, 1]T[ 5, 4]T[ 2, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 4, 2, 1, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              suux2ddxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          // qq' -> qq'q''qbar'' and qqbarqqbar' -> qqbarqqbar'q''qbar''
          else if (id1*id2 > 0) {
            if (pos1Sav == 1 && pos2Sav == 3) {
              // (q, -qbar', q', -qbar'', q'', qbar)
              Vec4toMGQCD(p, 1, 3, 2, 4, 0, 5, PNEW);
              icol = 1; // T[ 4, 6]T[ 3, 2]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 3, 2, 4, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 5) {
              // (q, -qbar', q', qbar'', q'', -qbar)
              Vec4toMGQCD(p, 1, 5, 2, 0, 4, 3, PNEW);
              icol = 2; // T[ 5, 2]T[ 3, 6]T[ 4, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 5, 2, 0, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 3 && pos2Sav == 5) {
              // (q, qbar', q', -qbar'', q'', -qbar)
              Vec4toMGQCD(p, 3, 5, 4, 0, 2, 1, PNEW);
              icol = 1; // T[ 4, 6]T[ 3, 2]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 5, 4, 0, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q, qbar', -q', qbar'', q'', qbar)
              Vec4toMGQCD(p, 0, 2, 5, 1, 3, 4, PNEW);
              icol = 2; // T[ 5, 2]T[ 3, 6]T[ 4, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 2, 5, 1, 3, 4};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q, qbar', q', qbar'', -q'', qbar)
              Vec4toMGQCD(p, 0, 4, 5, 3, 1, 2, PNEW);
              icol = 1; // T[ 4, 6]T[ 3, 2]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {0, 4, 5, 3, 1, 2};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q, qbar', -q', qbar'', -q'', qbar)
              Vec4toMGQCD(p, 2, 4, 1, 3, 5, 0, PNEW);
              icol = 2; // T[ 4, 1]T[ 5, 2]T[ 3, 6]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {2, 4, 1, 3, 5, 0};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sud2udssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          // qqbar' -> qqbar'q''qbar''
          else if (id1*id2 < 0) {
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q, -qbar', q', qbar'', q'', qbar)
              Vec4toMGQCD(p, 1, 0, 2, 5, 4, 3, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 0, 2, 5, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q, qbar', q', -qbar'', q'', qbar)
              Vec4toMGQCD(p, 3, 0, 4, 5, 2, 1, PNEW);
              icol = 1; // T[ 3, 4]T[ 2, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 0, 4, 5, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q, -qbar', q', qbar'', -q'', qbar)
              Vec4toMGQCD(p, 1, 4, 2, 3, 0, 5, PNEW);
              icol = 1; // T[ 3, 4]T[ 2, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {1, 4, 2, 3, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q, qbar', -q', -qbar'', q'', qbar)
              Vec4toMGQCD(p, 3, 2, 4, 1, 0, 5, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {3, 2, 4, 1, 0, 5};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 2 && pos2Sav == 5) {
              // (q, qbar', -q', qbar'', q'', -qbar)
              Vec4toMGQCD(p, 5, 2, 0, 1, 4, 3, PNEW);
              icol = 1; // T[ 3, 4]T[ 2, 6]T[ 5, 1]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 2, 0, 1, 4, 3};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else if (pos1Sav == 4 && pos2Sav == 5) {
              // (q, qbar', q', qbar'', -q'', -qbar)
              Vec4toMGQCD(p, 5, 4, 0, 3, 2, 1, PNEW);
              icol = 2; // T[ 2, 1]T[ 3, 6]T[ 5, 4]
              if (HEL[0] > 0) {
                int mapVtoMG[6] = {5, 4, 0, 3, 2, 1};
                for (int iMG=nMom-1; iMG>=0; --iMG) {
                  HEL[0] += nHels*(int)(pol[mapVtoMG[iMG]]+1)/2;
                  nHels *= 2;
                }
              }
              //cout<<"     icol = "<<icol<<", going for helicity amp "<<HEL[0]<<endl;
              sudx2udxssx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
            }
            else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          }
          
          else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
          
        }
        
        else cout<<" MG4interface : unrecognised colour order for 6-quark amplitude"<<endl;
        
      }
      
      // SIX-QUARK AMPLITUDES ------ END
      
      // Set masses back to original values
      coupl_mass_.bmass = bMassSav;
      coupl_mass_.cmass = cMassSav;
      coupl_mass_.smass = sMassSav;
      coupl_mass_.umass = uMassSav;
      coupl_mass_.dmass = dMassSav;
      
      // Add average over colours in the initial state
      if (id[pos1Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      if (id[pos2Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      
    }
    // end QCD 2->n
    //------------------------------------------------------------------------
    
    //------------------------------------------------------------------------
    // Z production, unpolarized only
    else if (nIn == 2 && id[nMom-1]==23) {
      
      // Massless only, setting all quark masses to zero
      double bMassSav   = coupl_mass_.bmass;
      coupl_mass_.bmass = 0.0;
      double cMassSav   = coupl_mass_.cmass;
      coupl_mass_.cmass = 0.0;
      double sMassSav   = coupl_mass_.smass;
      coupl_mass_.smass = 0.0;
      double dMassSav   = coupl_mass_.dmass;
      coupl_mass_.dmass = 0.0;
      double uMassSav   = coupl_mass_.umass;
      coupl_mass_.umass = 0.0;

      // Ensure ME evaluated on shell
      const double mSys = p[nMom-1].mCalc();
      coupl_mass_.zmass = mSys;
      
      // pp->Z
      if (nOut == 1) { if ((id[0]!=21) && (id[1]!=21) && (abs(id[0])==abs(id[1]))) {
        // Boost into rest frame of the Z
        p[2].e(mSys);     p[2].px(0.0); p[2].py(0.0); p[2].pz(0.0);
        p[0].e(mSys/2.0); p[0].px(0.0); p[0].py(0.0); p[0].pz(mSys/2.0);
        p[1].e(mSys/2.0); p[1].px(0.0); p[1].py(0.0); p[1].pz(-mSys/2.0);
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (abs(id[0]) % 2 == 0) swapZud();
        sddx2z_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        if (abs(id[0]) % 2 == 0) swapZud();
      } }
      
      // pp->jZ
      else if (nOut == 2) {
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        if (abs(id[0]) % 2 == 0) swapZud();
        if (pos1Sav == 0 && pos2Sav == 1) {
          // (-q, -g, qbar, Z)
          // MG order 1, 0, 2,
          Vec4toMG(p, 1, 0, 2, PNEW);
          sgdx2dxz_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2, 0]
        }
        else if (pos1Sav == 1 && pos2Sav == 2) {
          // (q, -g, -qbar, Z)
          // MG order 1, 2, 0,
          Vec4toMG(p, 1, 2, 0, PNEW);
          sgd2dz_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 2, 1, 0]
        }
        else if (pos1Sav == 0 && pos2Sav == 2) {
          // (-q, g, -qbar, Z)
          // MG order 2, 0, 1,
          Vec4toMG(p, 2, 0, 1, PNEW);
          sddx2gz_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 0, 2]
        }
        if (abs(id[0]) % 2 == 0) swapZud();
      }
      
      // pp->jjZ
      else if (nOut == 3) { 
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        
        // --------------------------------------------------------- 2 Quarks start
        if (nGluonTot == 2) { // momenta and ids are now given in colour order q g g qbar
          if (abs(id[0]) % 2 == 0) swapZud();
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, qbar, Z)
            // MG order 1, 0, 2, 3
            Vec4toMG(p, 1, 0, 2, 3, PNEW);
            sgd2gdz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 1, 2, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, qbar, Z)
            // MG order 2, 0, 1, 3
            Vec4toMG(p, 2, 0, 1, 3, PNEW);
            sgd2gdz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 1, 0, 2]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -qbar, Z)
            // MG order 3, 0, 1, 2
            Vec4toMG(p, 3, 0, 1, 2, PNEW);
            sddx2ggz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2, 3]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, qbar, Z)
            // MG order 1, 2, 0, 3
            Vec4toMG(p, 1, 2, 0, 3, PNEW);
            sgg2ddxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 1]
          }
          else if (pos1Sav== 1 && pos2Sav == 3) {
            // (q, -g, g, -qbar, Z)
            // MG order 1, 3, 2, 0
            Vec4toMG(p, 1, 3, 2, 0, PNEW);
            sgd2gdz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 1, 0, 2]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -qbar, Z)
            // MG order 2, 3, 1, 0
            Vec4toMG(p, 2, 3, 1, 0, PNEW);
            sgd2gdz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 1, 2, 0]
          }
          if (abs(id[0]) % 2 == 0) swapZud();
        }
        // --------------------------------------------------------- 2 Quarks end
        
        // --------------------------------------------------------- 4 Quarks start
        else if (nGluonTot == 0) { // momenta and ids are now given in order q qbar q qbar
          // Check if all quarks have the same flavour / same type (u/d)
          bool sameFlav = false;
          if ( (abs(id[0])==abs(id[1])) && (abs(id[0])==abs(id[2])) && (abs(id[0])==abs(id[3])) ) sameFlav = true;
          bool sameType = false;
          if ( (abs(id[0])%2==1) && (abs(id[1])%2==1) && (abs(id[2])%2==1) && (abs(id[3])%2==1) ) sameType = true;
          if ( (abs(id[0])%2==0) && (abs(id[1])%2==0) && (abs(id[2])%2==0) && (abs(id[3])%2==0) ) sameType = true;
          if (pos1Sav == 1 && pos2Sav == 3) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (q, -qbar, q, -qbar, Z)
            // MG order 1, 3, 2, 0
            Vec4toMG(p, 1, 3, 2, 0, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sds2dsz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            else if ( (!sameFlav) && (!sameType) ) sdu2duz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            else if (  (sameFlav) &&  (sameType) ) sdd2ddz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (-q, qbar, -q, qbar, Z)
            // MG order 0, 2, 3, 1
            Vec4toMG(p, 0, 2, 3, 1, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sds2dsz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            else if ( (!sameFlav) && (!sameType) ) sdu2duz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            else if (  (sameFlav) &&  (sameType) ) sdd2ddz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (-q, -qbar, q, qbar, Z)
            // MG order 1, 0, 2, 3
            Vec4toMG(p, 1, 0, 2, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdsx2dsxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else if ( (!sameFlav) && (!sameType) ) sdux2duxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (q, qbar, -q, -qbar, Z)
            // MG order 3, 2, 0, 1
            Vec4toMG(p, 3, 2, 0, 1, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdsx2dsxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else if ( (!sameFlav) && (!sameType) ) sdux2duxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (-q, qbar, q, -qbar, Z)
            // MG order 3, 0, 2, 1
            Vec4toMG(p, 3, 0, 2, 1, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sddx2ssxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else if ( (!sameFlav) && (!sameType) ) sddx2uuxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (q, -qbar, -q, qbar, Z)
            // MG order 1, 2, 0, 3
            Vec4toMG(p, 1, 2, 0, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sddx2ssxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else if ( (!sameFlav) && (!sameType) ) sddx2uuxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
        }
        // --------------------------------------------------------- 4 Quarks end
      }
      
      // pp->jjjZ
      else if (nOut == 4) { 
        int icol1 = 1; // different color flows
        int icol2 = 2;
        int icol3 = 3;
        int icol4 = 4;
        int icol5 = 5;
        int icol6 = 6;
        // Convert momenta to MadGraph format
        double PNEW[6][4];
        
        // --------------------------------------------------------- 2 Quarks start
        if (nGluonTot == 3) { // momenta and ids are now given in colour order q g g g qbar
          if (abs(id[0]) % 2 == 0) swapZud();
          if (pos1Sav == 0 && pos2Sav == 4) {
            // (-q, g, g, g, -qbar, Z)
            // MG order 4, 0, 1, 2, 3
            Vec4toMG(p, 4, 0, 1, 2, 3, PNEW);
            sddx2gggz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2, 3, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, g, qbar, Z)
            // MG order 1, 2, 0, 4, 3
            Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
            sgg2ddxgz_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 1, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -g, g, -g, qbar, Z)
            // MG order 1, 3, 0, 4, 2
            Vec4toMG(p, 1, 3, 0, 4, 2, PNEW);
            sgg2ddxgz_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 4, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -g, qbar, Z)
            // MG order 2, 3, 0, 4, 1
            Vec4toMG(p, 2, 3, 0, 4, 1, PNEW);
            sgg2ddxgz_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // T[ 2, 3, 4, 0, 1]
          }
          else if (pos1Sav == 3 && pos2Sav == 4) {
            // (q, g, g, -g, -qbar, Z)
            // MG order 4, 3, 0, 1, 2
            Vec4toMG(p, 4, 3, 0, 1, 2, PNEW);
            sdg2dggz_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // T[ 2, 0, 3, 4, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 4) {
            // (q, g, -g, g, -qbar, Z)
            // MG order 4, 2, 0, 3, 1
            Vec4toMG(p, 4, 2, 0, 3, 1, PNEW);
            sdg2dggz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 0, 4, 1, 3]
          }
          else if (pos1Sav == 1 && pos2Sav == 4) {
            // (q, -g, g, g, -qbar, Z)
            // MG order 4, 1, 0, 2, 3
            Vec4toMG(p, 4, 1, 0, 2, 3, PNEW);
            sdg2dggz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0, 1, 3, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, g, qbar, Z)
            // MG order 0, 1, 4, 3, 2
            Vec4toMG(p, 0, 1, 4, 3, 2, PNEW);
            sdg2dggz_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // T[ 2, 0, 3, 4, 1]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, g, qbar, Z)
            // MG order 0, 2, 4, 1, 3
            Vec4toMG(p, 0, 2, 4, 1, 3, PNEW);
            sdg2dggz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 0, 4, 1, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -g, qbar, Z)
            // MG order 0, 3, 4, 2, 1
            Vec4toMG(p, 0, 3, 4, 2, 1, PNEW);
            sdg2dggz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0, 1, 3, 4]
          }
          if (abs(id[0]) % 2 == 0) swapZud();
        }
        // --------------------------------------------------------- 2 Quarks end
        
        // --------------------------------------------------------- 4 Quarks start
        else if (nGluonTot == 1) { // momenta and ids are now given in colour order q qbar q g qbar
          // Check if all quarks have the same flavour / same type (u/d)
          vector<int> idsPosCheck; idsPosCheck.clear(); idsPosCheck.resize(0);
          for (int i=0; i<(int)id.size(); i++) if (id[i]<20) idsPosCheck.push_back(i);
          bool sameFlav = false;
          if ( (abs(idsPosCheck[0])==abs(idsPosCheck[1])) && (abs(idsPosCheck[0])==abs(idsPosCheck[2])) && 
               (abs(idsPosCheck[0])==abs(idsPosCheck[3])) ) sameFlav = true;
          bool sameType = false;
          if ( (abs(idsPosCheck[0])%2==1) && (abs(idsPosCheck[1])%2==1) && 
               (abs(idsPosCheck[2])%2==1) && (abs(idsPosCheck[2])%2==1) ) sameType = true;
          if ( (abs(idsPosCheck[0])%2==0) && (abs(idsPosCheck[1])%2==0) && 
               (abs(idsPosCheck[2])%2==0) && (abs(idsPosCheck[2])%2==0) ) sameType = true;
          if (pos1Sav == 1 && pos2Sav == 3) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (q, -qbar, q, -g, qbar, Z)
            // MG order 1, 3, 2, 0, 4
            Vec4toMG(p, 1, 3, 2, 0, 4, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdg2dssxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
            else if ( (!sameFlav) && (!sameType) ) sdg2duuxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
            else if (  (sameFlav) &&  (sameType) ) sdg2dddxz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 3 && pos2Sav == 4) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (q, qbar, q, -g, -qbar, Z)
            // MG order 4, 3, 0, 2, 1
            Vec4toMG(p, 4, 3, 0, 2, 1, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdg2dssxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
            else if ( (!sameFlav) && (!sameType) ) sdg2duuxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
            else if (  (sameFlav) &&  (sameType) ) sdg2dddxz_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (-q, qbar, q, -g, qbar, Z)
            // MG order 0, 3, 4, 2, 1
            Vec4toMG(p, 0, 3, 4, 2, 1, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdxg2dxssxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 4]T[ 3, 2, 1]
            else if ( (!sameFlav) && (!sameType) ) sdxg2dxuuxz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 4]T[ 3, 2, 1]
            else if (  (sameFlav) &&  (sameType) ) sdxg2dxddxz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 0, 4]T[ 3, 2, 1]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (q, qbar, -q, -g, qbar, Z)
            // MG order 2, 3, 1, 0, 4
            Vec4toMG(p, 2, 3, 1, 0, 4, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdxg2dxssxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
            else if ( (!sameFlav) && (!sameType) ) sdxg2dxuuxz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
            else if (  (sameFlav) &&  (sameType) ) sdxg2dxddxz_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 4) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (q, -qbar, -q, g, qbar, Z)
            // MG order 4, 0, 2, 1, 3
            Vec4toMG(p, 4, 0, 2, 1, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sddx2ssxgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
            else if ( (!sameFlav) && (!sameType) ) sddx2uuxgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxgz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
            // (q, -qbar, -q, g, qbar, Z)
            // MG order 1, 2, 0, 4, 3
            Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sddx2ssxgz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
            else if ( (!sameFlav) && (!sameType) ) sddx2uuxgz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxgz_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
            if (abs(id[pos1Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 2 && pos2Sav == 4) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (q, qbar, -q, g, -qbar, Z)
            // MG order 4, 2, 0, 1, 3
            Vec4toMG(p, 4, 2, 0, 1, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdsx2dsxgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
            else if ( (!sameFlav) && (!sameType) ) sdux2duxgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (-q, -qbar, q, g, qbar, Z)
            // MG order 1, 0, 2, 4, 3
            Vec4toMG(p, 1, 0, 2, 4, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sdsx2dsxgz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
            else if ( (!sameFlav) && (!sameType) ) sdux2duxgz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
            else if (  (sameFlav) &&  (sameType) ) sddx2ddxgz_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 1 && pos2Sav == 4) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (q, -qbar, q, g, -qbar, Z)
            // MG order 4, 1, 0, 2, 3
            Vec4toMG(p, 4, 1, 0, 2, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sds2dsgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            else if ( (!sameFlav) && (!sameType) ) sdu2dugz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            else if (  (sameFlav) &&  (sameType) ) sdd2ddgz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
            // (-q, qbar, -q, g, qbar, Z)
            // MG order 2, 0, 1, 4, 3
            Vec4toMG(p, 2, 0, 1, 4, 3, PNEW);
                 if ( (!sameFlav) &&  (sameType) ) sds2dsgz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            else if ( (!sameFlav) && (!sameType) ) sdu2dugz_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            else if (  (sameFlav) &&  (sameType) ) sdd2ddgz_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
            if (abs(id[pos2Sav]) % 2 == 0) swapZud();
          }
        }
        // --------------------------------------------------------- 4 Quarks end
      }
      
      // Set masses back to original values
      coupl_mass_.bmass = bMassSav;
      coupl_mass_.cmass = cMassSav;
      coupl_mass_.smass = sMassSav;
      coupl_mass_.umass = uMassSav;
      coupl_mass_.dmass = dMassSav;
    
      // Add average over colours in the initial state
      if (id[pos1Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      if (id[pos2Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      
    }
    // Z production
    //------------------------------------------------------------------------
  
    //------------------------------------------------------------------------
    // W production, unpolarized only
    else if (nIn == 2 && abs(id[nMom-1])==24) {
      
      // In case of W- reverse order of partons and use -ids
      if (id[nMom-1] < 0) {
        vector<int>  idsTmp = id;
        vector<Vec4> psTmp  = p;
        if (nOut == 4 && nGluonTot == 1) { // Special case
          map<int,int> newPos;
          newPos[0] = 1;
          newPos[1] = 0;
          newPos[2] = 4;
          newPos[3] = 3;
          newPos[4] = 2;
          for (map<int,int>::iterator it = newPos.begin(); it != newPos.end(); ++it) {
            p[it->first] = psTmp[it->second];
            if (id[it->first] != 21) id[it->first] = -idsTmp[it->second];
          }
          pos1Sav = newPos[pos1Sav];
          pos2Sav = newPos[pos2Sav];
          if (pos1Sav > pos2Sav) {
            int posTmp = pos1Sav;
            pos1Sav = pos2Sav;
            pos2Sav = posTmp;
          }
        } else {
          bool changedPos1 = false;
          bool changedPos2 = false;
          int pos1New = -1;
          int pos2New = -1;
          // Loop over all partons
          for (int i=0; i<(nMom-1); ++i) {
            p[i]  =  psTmp[nMom-2-i];
            id[i] = (idsTmp[nMom-2-i] == 21 ? 21 : -idsTmp[nMom-2-i]);
            if (pos1Sav == i && !changedPos1) {
              pos1New = nMom-2-i;
              changedPos1 = true;
            }
            if (pos2Sav == i && !changedPos2) {
              pos2New = nMom-2-i;
              changedPos2 = true;
            }
          }
          if (pos2New > pos1New) {
            pos1Sav = pos1New;
            pos2Sav = pos2New;
          } else {
            pos1Sav = pos2New;
            pos2Sav = pos1New;
          }
        }
      }
      
      // Get absolute values of ids
      vector<int> idAb = id;
      for (int i=0; i<(int)idAb.size(); i++) idAb[i] = abs(idAb[i]);
      
      // Massless only, setting all quark masses to zero
      double bMassSav   = coupl_mass_.bmass;
      coupl_mass_.bmass = 0.0;
      double cMassSav   = coupl_mass_.cmass;
      coupl_mass_.cmass = 0.0;
      double sMassSav   = coupl_mass_.smass;
      coupl_mass_.smass = 0.0;
      double dMassSav   = coupl_mass_.dmass;
      coupl_mass_.dmass = 0.0;
      double uMassSav   = coupl_mass_.umass;
      coupl_mass_.umass = 0.0;

      // Ensure ME evaluated on shell
      const double mSys = p[nMom-1].mCalc();
      coupl_mass_.wmass = mSys;
      
      // pp->W
      if (nOut == 1) { if ((id[0]!=21) && (id[1]!=21)) {
        // Boost into rest frame of the W
        p[2].e(mSys);     p[2].px(0.0); p[2].py(0.0); p[2].pz(0.0);
        p[0].e(mSys/2.0); p[0].px(0.0); p[0].py(0.0); p[0].pz(mSys/2.0);
        p[1].e(mSys/2.0); p[1].px(0.0); p[1].py(0.0); p[1].pz(-mSys/2.0);
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        setWcouplings(idAb[0], idAb[1]);
        sudx2wx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
      } }
      
      // pp->jW
      else if (nOut == 2) {
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        setWcouplings(idAb[0], idAb[2]);
        if (pos1Sav == 0 && pos2Sav == 1) {
          // (-q, -g, qbar, W)
          // MG order 1, 0, 2,
          Vec4toMG(p, 1, 0, 2, PNEW);
          sgdx2uxwx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2, 0]
        }
        else if (pos1Sav == 1 && pos2Sav == 2) {
          // (q, -g, -qbar, W)
          // MG order 2, 1, 0,
          Vec4toMG(p, 2, 1, 0, PNEW);
          sug2dwx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 2, 1, 0]
        }
        else if (pos1Sav == 0 && pos2Sav == 2) {
          // (-q, g, -qbar, W)
          // MG order 2, 0, 1,
          Vec4toMG(p, 2, 0, 1, PNEW);
          sudx2gwx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 0, 2]
        }
      }
      
      // pp->jjW
      else if (nOut == 3) { 
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        
        // --------------------------------------------------------- 2 Quarks start
        if (nGluonTot == 2) { // momenta and ids are now given in colour order q g g qbar
          setWcouplings(idAb[0], idAb[3]);
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, qbar, W)
            // MG order 1, 0, 2, 3
            Vec4toMG(p, 1, 0, 2, 3, PNEW);
            sgdx2guxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 3, 0, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, qbar, W)
            // MG order 2, 0, 1, 3
            Vec4toMG(p, 2, 0, 1, 3, PNEW);
            sgdx2guxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 3, 2, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -qbar, W)
            // MG order 3, 0, 1, 2
            Vec4toMG(p, 3, 0, 1, 2, PNEW);
            sudx2ggwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2, 3]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, qbar, W)
            // MG order 1, 2, 3, 0
            Vec4toMG(p, 1, 2, 3, 0, PNEW);
            sgg2uxdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 2, 0, 1]
          }
          else if (pos1Sav== 1 && pos2Sav == 3) {
            // (q, -g, g, -qbar, W)
            // MG order 3, 1, 2, 0
            Vec4toMG(p, 3, 1, 2, 0, PNEW);
            sug2gdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0, 1, 2]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -qbar, W)
            // MG order 3, 2, 1, 0
            Vec4toMG(p, 3, 2, 1, 0, PNEW);
            sug2gdwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0, 2, 1]
          }
        }
        // --------------------------------------------------------- 2 Quarks end
        
        // --------------------------------------------------------- 4 Quarks start
        else if (nGluonTot == 0) { // momenta and ids are now given in order q qbar q qbar
          // Count number of flavours
          map<int,int> flavours;
          for (int i=1; i<=5; ++i) flavours[i] = 0;
          for (int i=0; i<nMom; ++i) flavours[idAb[i]]++;
          int countFlavs = 0;
          for (int i=1; i<=5; ++i) if (flavours[i] > 0) countFlavs++;
          // 1) 3 different quark flavours, identify shower quark pair
          if (countFlavs == 3) {
            if (idAb[0] == idAb[3]) { // q qbar'' q' qbar
              setWcouplings(idAb[1], idAb[2]);
              // Swap the quark pairs -> q' qbar q qbar''
              vector<Vec4> psTmp = p;
              p[0] = psTmp[2];
              p[1] = psTmp[3];
              p[2] = psTmp[0]; 
              p[3] = psTmp[1];
              if (pos1Sav < 2) pos1Sav += 2;
              else             pos1Sav -= 2;
              if (pos2Sav > 1) pos2Sav -= 2;
              else             pos2Sav += 2;
              if (pos1Sav > pos2Sav) {
                int posTmp = pos1Sav;
                pos1Sav = pos2Sav;
                pos2Sav = posTmp;
              }
            } else if (idAb[1] == idAb[2]) { // q' qbar q qbar''
              setWcouplings(idAb[0], idAb[3]);
            } else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in pp -> Wjj with 3 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
            }
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q', -qbar, q, qbar'', W)
              // MG order 1, 0, 2, 3
              Vec4toMG(p, 1, 0, 2, 3, PNEW);
              scdx2cuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3]
            }
            else if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q', qbar, -q, qbar'', W)
              // MG order 2, 0, 1, 3
              Vec4toMG(p, 2, 0, 1, 3, PNEW);
              scxdx2cxuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3]
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q', qbar, q, -qbar'', W)
              // MG order 3, 0, 2, 1
              Vec4toMG(p, 3, 0, 2, 1, PNEW);
              sudx2ccxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // not possible
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q', -qbar, q, -qbar'', W)
              // MG order 1, 3, 2, 0
              Vec4toMG(p, 1, 3, 2, 0, PNEW);
              scu2cdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q', qbar, -q, -qbar'', W)
              // MG order 3, 2, 0, 1
              Vec4toMG(p, 3, 2, 0, 1, PNEW);
              sucx2dcxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3]
            }
          }
          // 2) 2 different quark flavours, identify single flavour
          else {
            bool singleFirst = false;
            bool swapPairs   = false;
            if ( (idAb[0] == idAb[1]) && (idAb[0] == idAb[2]) ) { // q qbar q qbar'
              setWcouplings(idAb[0], idAb[3]);
            } else if ( (idAb[0] == idAb[2]) && (idAb[0] == idAb[3]) ) { // q qbar' q qbar
              setWcouplings(idAb[0], idAb[1]);
              swapPairs   = true; // -> q qbar q qbar'
            } else if ( (idAb[1] == idAb[2]) && (idAb[1] == idAb[3]) ) { // q' qbar q qbar
              setWcouplings(idAb[0], idAb[3]);
              singleFirst = true;
            } else if ( (idAb[0] == idAb[1]) && (idAb[0] == idAb[3]) ) { // q qbar q' qbar
              setWcouplings(idAb[0], idAb[2]);
              singleFirst = true;
              swapPairs   = true; // q' qbar q qbar
            } else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in pp -> Wjj with 2 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
            }
            if (swapPairs) { // Swap the quark pairs
              vector<Vec4> psTmp = p;
              p[0] = psTmp[2];
              p[1] = psTmp[3];
              p[2] = psTmp[0]; 
              p[3] = psTmp[1]; 
              if (pos1Sav < 2) pos1Sav += 2;
              else             pos1Sav -= 2;
              if (pos2Sav > 1) pos2Sav -= 2;
              else             pos2Sav += 2;
              if (pos1Sav > pos2Sav) {
                int posTmp = pos1Sav;
                pos1Sav = pos2Sav;
                pos2Sav = posTmp;
              }
            }
            if (!singleFirst) { // q qbar q qbar'
              if (pos1Sav == 0 && pos2Sav == 1) {
                // (-q, -qbar, q, qbar', W)
                // MG order 1, 0, 2, 3
                Vec4toMG(p, 1, 0, 2, 3, PNEW);
                sddx2duxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3]
              }
              else if (pos1Sav == 0 && pos2Sav == 2) {
                // (-q, qbar, -q, qbar', W)
                // MG order 2, 0, 1, 3
                Vec4toMG(p, 2, 0, 1, 3, PNEW);
                sdxdx2dxuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3]
              }
              else if (pos1Sav == 0 && pos2Sav == 3) {
                // (-q, qbar, q, -qbar', W)
                // MG order 0, 3, 2, 1
                Vec4toMG(p, 0, 3, 2, 1, PNEW);
                sdxu2ddxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 0, 3]
              }
              else if (pos1Sav == 1 && pos2Sav == 2) {
                // (q, -qbar, -q, qbar', W)
                // MG order 1, 2, 0, 3
                Vec4toMG(p, 1, 2, 0, 3, PNEW);
                sddx2duxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
              }
              else if (pos1Sav == 1 && pos2Sav == 3) {
                // (q, -qbar, q, -qbar', W)
                // MG order 1, 3, 2, 0
                Vec4toMG(p, 1, 3, 2, 0, PNEW);
                sdu2ddwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
              }
              else if (pos1Sav == 2 && pos2Sav == 3) {
                // (q, qbar, -q, -qbar', W)
                // MG order 2, 3, 0, 1
                Vec4toMG(p, 2, 3, 0, 1, PNEW);
                sdxu2ddxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 1]T[ 2, 3]
              }
            } else { // q' qbar q qbar
              if (pos1Sav == 0 && pos2Sav == 1) {
                // (-q', -qbar, q, qbar, W)
                // MG order 1, 0, 2, 3
                Vec4toMG(p, 1, 0, 2, 3, PNEW);
                sudx2uuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3]
              }
              else if (pos1Sav == 0 && pos2Sav == 2) {
                // (-q', qbar, -q, qbar, W)
                // MG order 0, 2, 1, 3
                Vec4toMG(p, 0, 2, 1, 3, PNEW);
                sdxux2uxuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 2]T[ 1, 3]
              }
              else if (pos1Sav == 0 && pos2Sav == 3) {
                // (-q', qbar, q, -qbar, W)
                // MG order 3, 0, 2, 1
                Vec4toMG(p, 3, 0, 2, 1, PNEW);
                sudx2uuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
              }
              else if (pos1Sav == 1 && pos2Sav == 2) {
                // (q', -qbar, -q, qbar, W)
                // MG order 3, 0, 2, 1
                Vec4toMG(p, 3, 0, 2, 1, PNEW);
                suux2duxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
              }
              else if (pos1Sav == 1 && pos2Sav == 3) {
                // (q', -qbar, q, -qbar, W)
                // MG order 1, 3, 2, 0
                Vec4toMG(p, 1, 3, 2, 0, PNEW);
                suu2udwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
              }
              else if (pos1Sav == 2 && pos2Sav == 3) {
                // (q', qbar, -q, -qbar, W)
                // MG order 3, 2, 0, 1
                Vec4toMG(p, 3, 2, 0, 1, PNEW);
                suux2duxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 1]T[ 2, 3]
              }
            }
          }
        }
        // --------------------------------------------------------- 4 Quarks end
      }
      
      // pp->jjjW
      else if (nOut == 4) { 
        int icol1 = 1; // different color flows
        int icol2 = 2;
        int icol3 = 3;
        int icol4 = 4;
        int icol6 = 6;
        // Convert momenta to MadGraph format
        double PNEW[6][4];
        
        // --------------------------------------------------------- 2 Quarks start
        if (nGluonTot == 3) { // momenta and ids are now given in colour order q g g g qbar
          setWcouplings(idAb[0], idAb[4]);
          if (pos1Sav == 0 && pos2Sav == 4) {
            // (-q, g, g, g, -qbar, W)
            // MG order 4, 0, 1, 2, 3
            Vec4toMG(p, 4, 0, 1, 2, 3, PNEW);
            sudx2gggwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2, 3, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, g, qbar, W)
            // MG order 1, 2, 3, 4, 0
            Vec4toMG(p, 1, 2, 3, 4, 0, PNEW);
            sgg2guxdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 4, 3, 0, 1, 2]
          }
          else if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -g, g, -g, qbar, W)
            // MG order 1, 3, 2, 4, 0
            Vec4toMG(p, 1, 3, 2, 4, 0, PNEW);
            sgg2guxdwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 4, 3, 0, 2, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -g, qbar, W)
            // MG order 2, 3, 1, 4, 0
            Vec4toMG(p, 2, 3, 1, 4, 0, PNEW);
            sgg2guxdwx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 4, 3, 2, 0, 1]
          }
          else if (pos1Sav == 3 && pos2Sav == 4) {
            // (q, g, g, -g, -qbar, W)
            // MG order 4, 3, 1, 2, 0
            Vec4toMG(p, 4, 3, 1, 2, 0, PNEW);
            sug2ggdwx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 4, 0, 2, 3, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 4) {
            // (q, g, -g, g, -qbar, W)
            // MG order 4, 2, 3, 1, 0
            Vec4toMG(p, 4, 2, 3, 1, 0, PNEW);
            sug2ggdwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 4, 0, 3, 1, 2]
          }
          else if (pos1Sav == 1 && pos2Sav == 4) {
            // (q, -g, g, g, -qbar, W)
            // MG order 4, 1, 2, 3, 0
            Vec4toMG(p, 4, 1, 2, 3, 0, PNEW);
            sug2ggdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 4, 0, 1, 2, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, g, qbar, W)
            // MG order 1, 0, 2, 3, 4
            Vec4toMG(p, 1, 0, 2, 3, 4, PNEW);
            sgdx2gguxwx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 1, 4, 0, 2, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, g, qbar, W)
            // MG order 2, 0, 1, 3, 4
            Vec4toMG(p, 2, 0, 1, 3, 4, PNEW);
            sgdx2gguxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 4, 2, 0, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -g, qbar, W)
            // MG order 3, 0, 1, 2, 4
            Vec4toMG(p, 3, 0, 1, 2, 4, PNEW);
            sgdx2gguxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 4, 2, 3, 0]
          }
        }
        // --------------------------------------------------------- 2 Quarks end
        
        // --------------------------------------------------------- 4 Quarks start
        else if (nGluonTot == 1) { // momenta and ids are now given in colour order q qbar q g qbar
          // Count number of flavours
          map<int,int> flavours;
          for (int i=1; i<=5; ++i) flavours[i] = 0;
          for (int i=0; i<nMom; ++i) flavours[idAb[i]]++;
          int countFlavs = 0;
          for (int i=1; i<=5; ++i) if (flavours[i] > 0) countFlavs++;
          // 1) 3 different quark flavours, identify shower quark pair
          if (countFlavs == 3) {
            bool swapped = false;
            if (idAb[0] == idAb[4]) { // q qbar'' q' g qbar
              setWcouplings(idAb[1], idAb[2]);
              swapped = true;
              // Swap the quark pairs -> q' qbar q g qbar''
              vector<Vec4> psTmp = p;
              map<int,int> newPos;
              newPos[0] = 2;
              newPos[1] = 4;
              newPos[2] = 0;
              newPos[3] = 3;
              newPos[4] = 1;
              for (map<int,int>::iterator it = newPos.begin(); it != newPos.end(); ++it)
                p[it->first] = psTmp[it->second];
              pos1Sav = newPos[pos1Sav];
              pos2Sav = newPos[pos2Sav];
              if (pos1Sav > pos2Sav) {
                int posTmp = pos1Sav;
                pos1Sav = pos2Sav;
                pos2Sav = posTmp;
              }
            } else if (idAb[1] == idAb[2]) { // q' qbar q g qbar''
              setWcouplings(idAb[0], idAb[4]);
            } else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in pp -> Wjjj with 3 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
            }
            if (pos1Sav == 0 && pos2Sav == 1) {
              // (-q', -qbar, q, g, qbar'', W)
              // MG order 1, 0, 2, 4, 3
              Vec4toMG(p, 1, 0, 2, 4, 3, PNEW);
              if (swapped) scdx2cuxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
              else         scdx2cuxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
            }
            else if (pos1Sav == 0 && pos2Sav == 2) {
              // (-q', qbar, -q, g, qbar'', W)
              // MG order 2, 0, 1, 4, 3
              Vec4toMG(p, 2, 0, 1, 4, 3, PNEW);
              if (swapped) scxdx2cxuxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 1, 2, 4]
              else         scxdx2cxuxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3, 4]
            }
            else if (pos1Sav == 0 && pos2Sav == 3) {
              // (-q', qbar, q, -g, qbar'', W)
              // MG order 0, 3, 2, 1, 4
              Vec4toMG(p, 0, 3, 2, 1, 4, PNEW);
              if (swapped) sdxg2ccxuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 0, 3, 1]
              else         sdxg2ccxuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 2, 4, 1]
            }
            else if (pos1Sav == 0 && pos2Sav == 4) {
              // (-q', qbar, q, g, -qbar'', W)
              // MG order 0, 4, 2, 1, 3
              Vec4toMG(p, 0, 4, 2, 1, 3, PNEW);
              if (swapped) sdxu2ccxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 0, 3, 4]
              else         sdxu2ccxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 2, 1, 4]
            }
            else if (pos1Sav == 1 && pos2Sav == 2) {
              // not possible
            }
            else if (pos1Sav == 1 && pos2Sav == 3) {
              // (q', -qbar, q, -g, qbar'', W)
              // MG order 1, 3, 2, 0, 4
              Vec4toMG(p, 1, 3, 2, 0, 4, PNEW);
              if (swapped) scg2cduxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
              else         scg2cduxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
            }
            else if (pos1Sav == 1 && pos2Sav == 4) {
              // (q', -qbar, q, g, -qbar'', W)
              // MG order 1, 4, 2, 0, 3
              Vec4toMG(p, 1, 4, 2, 0, 3, PNEW);
              if (swapped) scu2cdgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1]T[ 3, 0, 4]
              else         scu2cdgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1, 4]
            }
            else if (pos1Sav == 2 && pos2Sav == 3) {
              // (q', qbar, -q, -g, qbar'', W)
              // MG order 2, 3, 1, 0, 4
              Vec4toMG(p, 2, 3, 1, 0, 4, PNEW);
              if (swapped) scxg2cxduxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 4]T[ 3, 2, 1]
              else         scxg2cxduxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
            }
            else if (pos1Sav == 2 && pos2Sav == 4) {
              // (q', qbar, -q, g, -qbar'', W)
              // MG order 2, 4, 1, 0, 3
              Vec4toMG(p, 2, 4, 1, 0, 3, PNEW);
              if (swapped) scxu2cxdgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 0, 1]T[ 3, 2, 4]
              else         scxu2cxdgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 1, 4]
            }
            else if (pos1Sav == 3 && pos2Sav == 4) {
              // (q', qbar, q, -g, -qbar'', W)
              // MG order 4, 3, 2, 1, 0
              Vec4toMG(p, 4, 3, 2, 1, 0, PNEW);
              if (swapped) sug2ccxdwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 4, 3, 1]
              else         sug2ccxdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 4, 3]T[ 2, 0, 1]
            }
          } 
          // 2) 2 different quark flavours, identify single flavour
          else {
            bool singleFirst = false;
            bool swapped     = false;
            if ( (idAb[0] == idAb[1]) && (idAb[0] == idAb[2]) ) { // q qbar q g qbar'
              setWcouplings(idAb[0], idAb[4]);
            } else if ( (idAb[0] == idAb[2]) && (idAb[0] == idAb[4]) ) { // q qbar' q g qbar
              setWcouplings(idAb[0], idAb[1]);
              swapped     = true; // -> q qbar q g qbar'
            } else if ( (idAb[1] == idAb[2]) && (idAb[1] == idAb[4]) ) { // q' qbar q g qbar
              setWcouplings(idAb[0], idAb[1]);
              singleFirst = true;
            } else if ( (idAb[0] == idAb[1]) && (idAb[0] == idAb[4]) ) { // q qbar q' g qbar
              setWcouplings(idAb[0], idAb[2]);
              singleFirst = true;
              swapped     = true; // q' qbar q g qbar
            } else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in pp -> Wjjj with 2 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
            }
            if (swapped) { // Swap the quark pairs
              vector<Vec4> psTmp = p;
              map<int,int> newPos;
              newPos[0] = 2;
              newPos[1] = 4;
              newPos[2] = 0;
              newPos[3] = 3;
              newPos[4] = 1;
              for (map<int,int>::iterator it = newPos.begin(); it != newPos.end(); ++it)
                p[it->first] = psTmp[it->second];
              pos1Sav = newPos[pos1Sav];
              pos2Sav = newPos[pos2Sav];
              if (pos1Sav > pos2Sav) {
                int posTmp = pos1Sav;
                pos1Sav = pos2Sav;
                pos2Sav = posTmp;
              }
            }
            if (!singleFirst) { // q qbar q g qbar'
              if (pos1Sav == 0 && pos2Sav == 1) {
                // (-q, -qbar, q, g, qbar', W)
                // MG order 1, 0, 2, 4, 3
                Vec4toMG(p, 1, 0, 2, 4, 3, PNEW);
                if (swapped) sddx2duxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
                else         sddx2duxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
              }
              else if (pos1Sav == 0 && pos2Sav == 2) {
                // (-q, qbar, -q, g, qbar', W)
                // MG order 2, 0, 1, 4, 3
                Vec4toMG(p, 2, 0, 1, 4, 3, PNEW);
                if (swapped) sdxdx2dxuxgwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 1, 2, 4]
                else         sdxdx2dxuxgwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3, 4]
              }
              else if (pos1Sav == 0 && pos2Sav == 3) {
                // (-q, qbar, q, -g, qbar', W)
                // MG order 0, 3, 2, 1, 4
                Vec4toMG(p, 0, 3, 2, 1, 4, PNEW);
                if (swapped) sdxg2ddxuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 0, 3, 1]
                else         sdxg2ddxuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 2, 4, 1]
              }
              else if (pos1Sav == 0 && pos2Sav == 4) {
                // (-q, qbar, q, g, -qbar', W)
                // MG order 4, 0, 2, 1, 3
                Vec4toMG(p, 4, 0, 2, 1, 3, PNEW);
                if (swapped) sudx2ddxgwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
                else         sudx2ddxgwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
              }
              else if (pos1Sav == 1 && pos2Sav == 2) {
                // (q, -qbar, -q, g, qbar', W)
                // MG order 1, 2, 0, 4, 3
                Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
                if (swapped) sddx2duxgwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
                else         sddx2duxgwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
              }
              else if (pos1Sav == 1 && pos2Sav == 3) {
                // (q, -qbar, q, -g, qbar', W)
                // MG order 1, 3, 0, 2, 4
                Vec4toMG(p, 1, 3, 0, 2, 4, PNEW);
                if (swapped) sdg2dduxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 4]T[ 2, 0, 1]
                else         sdg2dduxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 3, 4, 1]
              }
              else if (pos1Sav == 1 && pos2Sav == 4) {
                // (q, -qbar, q, g, -qbar', W)
                // MG order 1, 4, 0, 2, 3
                Vec4toMG(p, 1, 4, 0, 2, 3, PNEW);
                if (swapped) sdu2ddgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 1]T[ 2, 0, 4]
                else         sdu2ddgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 3, 1, 4]
              }
              else if (pos1Sav == 2 && pos2Sav == 3) {
                // (q, qbar, -q, -g, qbar', W)
                // MG order 2, 3, 0, 1, 4
                Vec4toMG(p, 2, 3, 0, 1, 4, PNEW);
                if (swapped) sdxg2ddxuxwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 0, 4]T[ 2, 3, 1]
                else         sdxg2ddxuxwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 0, 4, 1]
              }
              else if (pos1Sav == 2 && pos2Sav == 4) {
                // (q, qbar, -q, g, -qbar', W)
                // MG order 4, 2, 0, 1, 3
                Vec4toMG(p, 4, 2, 0, 1, 3, PNEW);
                if (swapped) sudx2ddxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 4]T[ 2, 3, 4]
                else         sudx2ddxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
              }
              else if (pos1Sav == 3 && pos2Sav == 4) {
                // (q, qbar, q, -g, -qbar', W)
                // MG order 4, 3, 0, 2, 1
                Vec4toMG(p, 4, 3, 0, 2, 1, PNEW);
                if (swapped) sug2dddxwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
                else         sug2dddxwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
              }
            } else { // q' qbar q g qbar
              if (pos1Sav == 0 && pos2Sav == 1) {
                // (-q', -qbar, q, g, qbar, W)
                // MG order 1, 0, 2, 4, 3
                Vec4toMG(p, 1, 0, 2, 4, 3, PNEW);
                if (swapped) sudx2uuxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
                else         sudx2uuxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
              }
              else if (pos1Sav == 0 && pos2Sav == 2) {
                // (-q', qbar, -q, g, qbar, W)
                // MG order 0, 2, 1, 4, 3
                Vec4toMG(p, 0, 2, 1, 4, 3, PNEW);
                if (swapped) sdxux2uxuxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 0, 2, 4]
                else         sdxux2uxuxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 0, 2]T[ 1, 3, 4]
              }
              else if (pos1Sav == 0 && pos2Sav == 3) {
                // (-q', qbar, q, -g, qbar, W)
                // MG order 3, 0, 2, 1, 4
                Vec4toMG(p, 3, 0, 2, 1, 4, PNEW);
                if (swapped) sgdx2uuxuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 1, 3, 0]
                else         sgdx2uuxuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 4, 0]
              }
              else if (pos1Sav == 0 && pos2Sav == 4) {
                // (-q', qbar, q, g, -qbar, W)
                // MG order 4, 0, 2, 1, 3
                Vec4toMG(p, 4, 0, 2, 1, 3, PNEW);
                if (swapped) sudx2uuxgwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
                else         sudx2uuxgwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
              }
              else if (pos1Sav == 1 && pos2Sav == 2) {
                // (q', -qbar, -q, g, qbar, W)
                // MG order 1, 2, 0, 4, 3
                Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
                if (swapped) suux2duxgwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 2, 0, 4]
                else         suux2duxgwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
              }
              else if (pos1Sav == 1 && pos2Sav == 3) {
                // (q', -qbar, q, -g, qbar, W)
              // MG order 1, 3, 0, 2, 4
                Vec4toMG(p, 1, 3, 0, 2, 4, PNEW);
                if (swapped) sug2duuxwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 4]T[ 2, 0, 1]
                else         sug2duuxwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 3, 4, 1]
              }
              else if (pos1Sav == 1 && pos2Sav == 4) {
                // (q', -qbar, q, g, -qbar, W)
                // MG order 1, 4, 0, 2, 3
                Vec4toMG(p, 1, 4, 0, 2, 3, PNEW);
                if (swapped) suu2dugwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 1]T[ 2, 0, 4]
                else         suu2dugwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 3, 1, 4]
              }
              else if (pos1Sav == 2 && pos2Sav == 3) {
                // (q', qbar, -q, -g, qbar, W)
                // MG order 3, 2, 1, 4, 0
                Vec4toMG(p, 3, 2, 1, 4, 0, PNEW);
                if (swapped) sgux2uxuxdwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 3]T[ 4, 2, 0]
                else         sgux2uxuxdwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 4, 2]T[ 1, 3, 0]
              }
              else if (pos1Sav == 2 && pos2Sav == 4) {
                // (q', qbar, -q, g, -qbar, W)
                // MG order 4, 2, 0, 1, 3
                Vec4toMG(p, 4, 2, 0, 1, 3, PNEW);
                if (swapped) suux2duxgwx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
                else         suux2duxgwx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
              }
              else if (pos1Sav == 3 && pos2Sav == 4) {
                // (q', qbar, q, -g, -qbar, W)
                // MG order 4, 3, 0, 2, 1
                Vec4toMG(p, 4, 3, 0, 2, 1, PNEW);
                if (swapped) sug2duuxwx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
                else         sug2duuxwx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
              }
            }
          }
        }
        // --------------------------------------------------------- 4 Quarks end
      }
      
      // Set masses back to original values
      coupl_mass_.bmass = bMassSav;
      coupl_mass_.cmass = cMassSav;
      coupl_mass_.smass = sMassSav;
      coupl_mass_.umass = uMassSav;
      coupl_mass_.dmass = dMassSav;
    
      // Add average over colours in the initial state
      if (id[pos1Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      if (id[pos2Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      
    }
    // W production
    //------------------------------------------------------------------------
  
    //------------------------------------------------------------------------
    // H production, unpolarized only
    else if (nIn == 2 && id[nMom-1]==25) {
      
      // Massless only, setting all quark masses to zero
      double bMassSav   = coupl_mass_.bmass;
      coupl_mass_.bmass = 0.0;
      double cMassSav   = coupl_mass_.cmass;
      coupl_mass_.cmass = 0.0;
      double sMassSav   = coupl_mass_.smass;
      coupl_mass_.smass = 0.0;
      double dMassSav   = coupl_mass_.dmass;
      coupl_mass_.dmass = 0.0;
      double uMassSav   = coupl_mass_.umass;
      coupl_mass_.umass = 0.0;

      // Ensure ME evaluated on shell
      const double mSys = p[nMom-1].mCalc();
      coupl_mass_.hmass = mSys;
      
      // pp->H
      if (nOut == 1) {
        // Boost into rest frame of the H
        p[2].e(mSys);     p[2].px(0.0); p[2].py(0.0); p[2].pz(0.0);
        p[0].e(mSys/2.0); p[0].px(0.0); p[0].py(0.0); p[0].pz(mSys/2.0);
        p[1].e(mSys/2.0); p[1].px(0.0); p[1].py(0.0); p[1].pz(-mSys/2.0);
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 2) sgg2h_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        else {
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          sbbx2h_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }
      }
      
      // pp->jH
      else if (nOut == 2) {
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        if (nGluonTot == 3) {
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-g, -g, g, H)
            // MG order 0, 1, 2,
            Vec4toMG(p, 0, 1, 2, PNEW);
            sgg2gh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // F[0, 1, 2]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-g, g, -g, H)
            // MG order 0, 2, 1,
            Vec4toMG(p, 0, 2, 1, PNEW);
            sgg2gh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // F[0, 2, 1]
          }
        }
        else if (nGluonTot == 1) {
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -qbar, H)
            // MG order 1, 2, 0,
            Vec4toMG(p, 1, 2, 0, PNEW);
            sgb2bh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, qbar, H)
            // MG order 1, 0, 2,
            Vec4toMG(p, 1, 0, 2, PNEW);
            sgbx2bxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -qbar, H)
            // MG order 2, 0, 1,
            Vec4toMG(p, 2, 0, 1, PNEW);
            sbbx2gh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2]
          }
        }
      }
      
      // pp->jjH
      else if (nOut == 3) { 
        int icol1 = 1; // different color flows
        int icol2 = 2;
        int icol5 = 5;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        
        // --------------------------------------------------------- 4 Gluons start
        if (nGluonTot == 4) { // momenta and ids are now given in order g(i) g g g
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-g, -g, g, g, H)
            // MG order 0, 1, 2, 3,
            Vec4toMG(p, 0, 1, 2, 3, PNEW);
            sgg2ggh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // F[0, 1, 2, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-g, g, -g, g, H)
            // MG order 0, 2, 1, 3,
            Vec4toMG(p, 0, 2, 1, 3, PNEW);
            sgg2ggh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // F[0, 2, 1, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-g, g, g, -g, H)
            // MG order 0, 3, 1, 2,
            Vec4toMG(p, 0, 3, 1, 2, PNEW);
            sgg2ggh_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // F[0, 2, 3, 1]
          }
        }
        // --------------------------------------------------------- 4 Gluons end
        
        // --------------------------------------------------------- 2 Gluons start
        else if (nGluonTot == 2) { // momenta and ids are now given in colour order q g g qbar
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, qbar, H)
            // MG order 1, 2, 0, 3,
            Vec4toMG(p, 1, 2, 0, 3, PNEW);
            sgg2bbxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 1]
          }
          else if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -g, g, -qbar, H)
            // MG order 1, 3, 0, 2,
            Vec4toMG(p, 1, 3, 0, 2, PNEW);
            sgb2bgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 1, 0, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, qbar, H)
            // MG order 1, 0, 3, 2,
            Vec4toMG(p, 1, 0, 3, 2, PNEW);
            sgbx2bxgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2, 0, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, qbar, H)
            // MG order 2, 0, 3, 1,
            Vec4toMG(p, 2, 0, 3, 1, PNEW);
            sgbx2bxgh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2, 3, 0]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -qbar, H)
            // MG order 2, 3, 0, 1,
            Vec4toMG(p, 2, 3, 0, 1, PNEW);
            sgb2bgh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1, 3, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -qbar, H)
            // MG order 3, 0, 2, 1,
            Vec4toMG(p, 3, 0, 2, 1, PNEW);
            sbbx2ggh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0, 3, 2]
          }
        }
        // --------------------------------------------------------- 2 Gluons end
        
        // --------------------------------------------------------- 0 Gluons start
        else if (nGluonTot == 0) { // momenta and ids are now given in order q qbar q qbar
          // Check if all quarks have the same flavour
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) ) sameFlav = true;
          int posM1 = pos1Sav;
          int posM2 = pos2Sav;
          if (!sameFlav) {
            if ( (pos1Sav == 0 && pos2Sav == 1) || (pos1Sav == 2 && pos2Sav == 3) ) { posM1 = pos2Sav; posM2 = pos1Sav; }
            else if (pos1Sav == 0 && pos2Sav == 3) { posM1 = 0; posM2 = 2; }
            else if (pos1Sav == 1 && pos2Sav == 2) { posM1 = 2; posM2 = 0; }
          }
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[posM1])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[posM1])] );
          coupl_yuk_.ghcha[0] = complex<double>( 0.0, yukCoup[abs(id[posM2])] );
          coupl_yuk_.ghcha[1] = complex<double>( 0.0, yukCoup[abs(id[posM2])] );
          if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -qbar, q, -qbar, H)
            // MG order 1, 3, 2, 0
            Vec4toMG(p, 1, 3, 2, 0, PNEW);
            if (sameFlav) sbb2bbh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
            else          sbc2bch_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, qbar, -q, qbar, H)
            // MG order 0, 2, 3, 1
            Vec4toMG(p, 0, 2, 3, 1, PNEW);
            if (sameFlav) sbxbx2bxbxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3]
            else          sbxcx2bxcxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 0, 3]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -qbar, q, qbar, H)
            // MG order 1, 0, 2, 3
            Vec4toMG(p, 1, 0, 2, 3, PNEW);
            if (sameFlav) sbbx2bbxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else          sbcx2bcxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, qbar, -q, -qbar, H)
            // MG order 3, 2, 0, 1
            Vec4toMG(p, 3, 2, 0, 1, PNEW);
            if (sameFlav) sbbx2bbxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
            else          sbcx2bcxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, qbar, q, -qbar, H)
            // MG order 3, 0, 2, 1
            Vec4toMG(p, 3, 0, 2, 1, PNEW);
            if (sameFlav) sbbx2bbxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else          sbbx2ccxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -qbar, -q, qbar, H)
            // MG order 1, 2, 0, 3
            Vec4toMG(p, 1, 2, 0, 3, PNEW);
            if (sameFlav) sbbx2bbxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
            else          sbbx2ccxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3]
          }
        }
        // --------------------------------------------------------- 0 Gluons end
      }
      
      // pp->jjjH
      else if (nOut == 4) { 
        int icol1  = 1; // different color flows
        int icol2  = 2;
        int icol3  = 3;
        int icol4  = 4;
        int icol5  = 5;
        int icol6  = 6;
        int icol18 = 18;
        int icol22 = 22;
        double PNEW[6][4];
        
        // --------------------------------------------------------- 5 Gluons start
        if (nGluonTot == 5) { // momenta and ids are now given in order g(i) g g g g
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-g, -g, g, g, g, H)
            // MG order 0, 1, 2, 3, 4,
            Vec4toMG(p, 0, 1, 2, 3, 4, PNEW);
            sgg2gggh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // F[0, 1, 2, 3, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-g, g, -g, g, g, H)
            // MG order 0, 2, 1, 3, 4,
            Vec4toMG(p, 0, 2, 1, 3, 4, PNEW);
            sgg2gggh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // F[0, 2, 1, 3, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-g, g, g, -g, g, H)
            // MG order 0, 3, 1, 2, 4,
            Vec4toMG(p, 0, 3, 1, 2, 4, PNEW);
            sgg2gggh_(&colorDepth,&icol18,PNEW,HEL,&MGME2); // F[0, 2, 3, 1, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 4) {
            // (-g, g, g, g, -g, H)
            // MG order 0, 4, 1, 2, 3,
            Vec4toMG(p, 0, 4, 1, 2, 3, PNEW);
            sgg2gggh_(&colorDepth,&icol22,PNEW,HEL,&MGME2); // F[0, 2, 3, 4, 1]
          }
        }
        // --------------------------------------------------------- 5 Gluons end
        
        // --------------------------------------------------------- 3 Gluons start
        else if (nGluonTot == 3) { // momenta and ids are now given in colour order q g g g qbar
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[0])] );
          if (pos1Sav == 0 && pos2Sav == 4) {
            // (-q, g, g, g, -qbar, H)
            // MG order 4, 0, 1, 2, 3,
            Vec4toMG(p, 4, 0, 1, 2, 3, PNEW);
            sbbx2gggh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 0, 2, 3, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -g, -g, g, qbar, H)
            // MG order 1, 2, 0, 4, 3,
            Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
            sgg2bbxgh_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 1, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -g, g, -g, qbar, H)
            // MG order 1, 3, 0, 4, 2,
            Vec4toMG(p, 1, 3, 0, 4, 2, PNEW);
            sgg2bbxgh_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // T[ 2, 3, 0, 4, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, g, -g, -g, qbar, H)
            // MG order 2, 3, 0, 4, 1,
            Vec4toMG(p, 2, 3, 0, 4, 1, PNEW);
            sgg2bbxgh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 3, 4, 0, 1]
          }
          else if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -g, g, g, qbar, H)
            // MG order 1, 0, 4, 2, 3,
            Vec4toMG(p, 1, 0, 4, 2, 3, PNEW);
            sgbx2bxggh_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 1, 2, 0, 3, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, g, -g, g, qbar, H)
            // MG order 2, 0, 4, 1, 3,
            Vec4toMG(p, 2, 0, 4, 1, 3, PNEW);
            sgbx2bxggh_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2, 3, 0, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, g, g, -g, qbar, H)
            // MG order 3, 0, 4, 1, 2,
            Vec4toMG(p, 3, 0, 4, 1, 2, PNEW);
            sgbx2bxggh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2, 3, 4, 0]
          }
          else if (pos1Sav == 1 && pos2Sav == 4) {
            // (q, -g, g, g, -qbar, H)
            // MG order 1, 4, 0, 2, 3,
            Vec4toMG(p, 1, 4, 0, 2, 3, PNEW);
            sgb2bggh_(&colorDepth,&icol5,PNEW,HEL,&MGME2); // T[ 2, 1, 0, 3, 4] 
          }
          else if (pos1Sav == 2 && pos2Sav == 4) {
            // (q, g, -g, g, -qbar, H)
            // MG order 2, 4, 0, 1, 3,
            Vec4toMG(p, 2, 4, 0, 1, 3, PNEW);
            sgb2bggh_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 1, 3, 0, 4]
          }
          else if (pos1Sav == 3 && pos2Sav == 4) {
            // (q, g, g, -g, -qbar, H)
            // MG order 3, 4, 0, 1, 2,
            Vec4toMG(p, 3, 4, 0, 1, 2, PNEW);
            sgb2bggh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 1, 3, 4, 0]
          }
        }
        // --------------------------------------------------------- 3 Gluons end
        
        // --------------------------------------------------------- 1 Gluon start
        else if (nGluonTot == 1) { // momenta and ids are now given in order q qbar q g qbar
          // Check if all quarks have the same flavour
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) ) sameFlav = true;
          int posM1 = pos1Sav;
          int posM2 = pos2Sav;
          if (!sameFlav) {
            if ( (pos1Sav == 0 && pos2Sav == 1) || (pos1Sav == 2 && pos2Sav == 4) ) { posM1 = pos2Sav; posM2 = pos1Sav; }
            else if ( (pos1Sav == 0 && pos2Sav == 4) || (pos1Sav == 0 && pos2Sav == 3) || (pos1Sav == 3 && pos2Sav == 4) ) { posM1 = 0; posM2 = 2; }
            else if ( (pos1Sav == 1 && pos2Sav == 2) || (pos1Sav == 1 && pos2Sav == 3) || (pos1Sav == 2 && pos2Sav == 3) ) { posM1 = 2; posM2 = 0; }
          }
          else {
            posM1 = (id[pos1Sav]==21 ? pos2Sav : pos1Sav);
            posM2 = (id[pos2Sav]==21 ? pos1Sav : pos2Sav);
          }
          coupl_yuk_.ghbot[0] = complex<double>( 0.0, yukCoup[abs(id[posM1])] );
          coupl_yuk_.ghbot[1] = complex<double>( 0.0, yukCoup[abs(id[posM1])] );
          coupl_yuk_.ghcha[0] = complex<double>( 0.0, yukCoup[abs(id[posM2])] );
          coupl_yuk_.ghcha[1] = complex<double>( 0.0, yukCoup[abs(id[posM2])] );
          if (pos1Sav == 0 && pos2Sav == 1) {
            // (-q, -qbar, q, g, qbar, H)
            // MG order 1, 0, 2, 4, 3,
            Vec4toMG(p, 1, 0, 2, 4, 3, PNEW);
            if (sameFlav) sbbx2bbxgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
            else          sbcx2bcxgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 0]T[ 2, 3, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 2) {
            // (-q, qbar, -q, g, qbar, H)
            // MG order 0, 2, 4, 1, 3,
            Vec4toMG(p, 0, 2, 4, 1, 3, PNEW);
            if (sameFlav) sbxbx2bxbxgh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 1, 2, 4]
            else          sbxcx2bxcxgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 0, 3]T[ 1, 2, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 3) {
            // (-q, qbar, q, -g, qbar, H)
            // MG order 0, 3, 4, 2, 1,
            Vec4toMG(p, 0, 3, 4, 2, 1, PNEW);
            if (sameFlav) sbxg2bxbbxh_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 3, 2, 1]T[ 0, 4]
            else          sbxg2bxccxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 2, 1]T[ 0, 4]
          }
          else if (pos1Sav == 0 && pos2Sav == 4) {
            // (-q, qbar, q, g, -qbar, H)
            // MG order 4, 0, 2, 1, 3,
            Vec4toMG(p, 4, 0, 2, 1, 3, PNEW);
            if (sameFlav) sbbx2bbxgh_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 2, 0, 4]T[ 1, 3]
            else          sbbx2ccxgh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 0, 4]T[ 1, 3]
          }
          else if (pos1Sav == 1 && pos2Sav == 2) {
            // (q, -qbar, -q, g, qbar, H)
            // MG order 1, 2, 0, 4, 3,
            Vec4toMG(p, 1, 2, 0, 4, 3, PNEW);
            if (sameFlav) sbbx2bbxgh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
            else          sbbx2ccxgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 0]T[ 1, 3, 4]
          }
          else if (pos1Sav == 1 && pos2Sav == 3) {
            // (q, -qbar, q, -g, qbar, H)
            // MG order 1, 3, 2, 0, 4,
            Vec4toMG(p, 1, 3, 2, 0, 4, PNEW);
            if (sameFlav) sbg2bbbxh_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
            else          sbg2bccxh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 4, 1]
          }
          else if (pos1Sav == 1 && pos2Sav == 4) {
            // (q, -qbar, q, g, -qbar, H)
            // MG order 1, 4, 2, 0, 3,
            Vec4toMG(p, 1, 4, 2, 0, 3, PNEW);
            if (sameFlav) sbb2bbgh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1, 4]
            else          sbc2bcgh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 0]T[ 2, 1, 4]
          }
          else if (pos1Sav == 2 && pos2Sav == 3) {
            // (q, qbar, -q, -g, qbar, H)
            // MG order 2, 3, 1, 0, 4,
            Vec4toMG(p, 2, 3, 1, 0, 4, PNEW);
            if (sameFlav) sbxg2bxbbxh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
            else          sbxg2bxccxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 2]T[ 0, 4, 1]
          }
          else if (pos1Sav == 2 && pos2Sav == 4) {
            // (q, qbar, -q, g, -qbar, H)
            // MG order 4, 2, 0, 1, 3,
            Vec4toMG(p, 4, 2, 0, 1, 3, PNEW);
            if (sameFlav) sbbx2bbxgh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
            else          sbcx2bcxgh_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 2, 3]T[ 1, 0, 4]
          }
          else if (pos1Sav == 3 && pos2Sav == 4) {
            // (q, qbar, q, -g, -qbar, H)
            // MG order 4, 3, 0, 2, 1,
            Vec4toMG(p, 4, 3, 0, 2, 1, PNEW);
            if (sameFlav) sbg2bbbxh_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
            else          sbg2bccxh_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 2, 4]T[ 3, 0, 1]
          }
        }
        // --------------------------------------------------------- 1 Gluon end
      }
      
      // Set masses back to original values
      coupl_mass_.bmass = bMassSav;
      coupl_mass_.cmass = cMassSav;
      coupl_mass_.smass = sMassSav;
      coupl_mass_.umass = uMassSav;
      coupl_mass_.dmass = dMassSav;
    
      // Add average over colours in the initial state
      if (id[pos1Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      if (id[pos2Sav]==21) MGME2*=1.0/8.0;
      else MGME2*=1.0/3.0;
      
    }
    // H production
    //------------------------------------------------------------------------
    
    //------------------------------------------------------------------------
    // Z decay
    else if (nIn == 1 && id0 == 23) {
      
      // Polarized?
      if (isPolarised) HEL[0] = 1;
      nPolIn = 3; // Default: sum over all incoming and outgoing polarizations

      // Ensure ME evaluated on shell
      const double mSys = p[0].mCalc();
      coupl_mass_.zmass = mSys;
      
      // Z->jj
      if (nOut == 2) { if ((id[1]!=21) && (id[2]!=21) && (abs(id[1])==abs(id[2]))) {
        // 0) Default assumption: fully unpolarized Z decay
        // 1) Sum over Z polarizations, for specific final-state polarizations
        if (pol[0] == 9 && pol[1] != 9 && pol[2] != 9) {
          HEL[1] = (int)(1 + (pol[2] + 1)/2 + (pol[1] + 1));
          HEL[2] = (int)(5 + (pol[2] + 1)/2 + (pol[1] + 1));
          HEL[3] = (int)(9 + (pol[2] + 1)/2 + (pol[1] + 1));
        }
        // 2) Fully polarized Z decay (nPolIn = 1 in normalization)
        else if (pol[0] != 9 && pol[1] != 9 && pol[2] != 9) {
          nPolIn = 1;
          HEL[1] = (int)(1 + (pol[0] + 1)*4 + (pol[2] + 1)/2 + (pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        coupl_mass_.dmass = m[1];
        if (abs(id[1]) % 2 == 0) swapZud();
        sz2ddx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        if (abs(id[1]) % 2 == 0) swapZud();
      } }
      
      // Z->jjj
      else if (nOut == 3) {
        // 0) Default assumption: fully unpolarized Z decay
        // 1) Sum over Z polarizations, for specific final-state polarizations
        if (pol[0] == 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9) {
          HEL[1] = (int)(1 + (pol[3] + 1)/2 + (pol[2] + 1) + 2*(pol[1] + 1));
          HEL[2] = (int)(9 + (pol[3] + 1)/2 + (pol[2] + 1) + 2*(pol[1] + 1));
          HEL[3] = (int)(17 + (pol[3] + 1)/2 + (pol[2] + 1) + 2*(pol[1] + 1));
        }
        // 2) Fully polarized Z decay (nPolIn = 1 for normalization)
        else if (pol[0] != 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9) {
          nPolIn = 1;
          HEL[1] = (int)(1 + (pol[0] + 1)*8 + (pol[3] + 1)/2 + (pol[2] + 1) 
                         + 2*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        coupl_mass_.dmass = m[1];
        if (abs(id[1]) % 2 == 0) swapZud();
        sz2dgdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 3, 2]
        if (abs(id[1]) % 2 == 0) swapZud();
      }
      
      // Z->jjjj
      else if (nOut == 4) {
        // 0) Default assumption: fully unpolarized Z decay
        // 1) Sum over Z polarizations, for specific final-state polarizations
        if (pol[0] == 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 && pol[4] != 9) {
          HEL[1] = (int)(1 + (pol[4] + 1)/2 + (pol[3] + 1) + 2*(pol[2] + 1) 
                          + 4*(pol[1] + 1));
          HEL[2] = (int)(17 + (pol[4] + 1)/2 + (pol[3] + 1) + 2*(pol[2] + 1) 
                         + 4*(pol[1] + 1));
          HEL[3] = (int)(33 + (pol[4] + 1)/2 + (pol[3] + 1) + 2*(pol[2] + 1) 
                         + 4*(pol[1] + 1));
        }
        // 2) Fully polarized Z decay (-> nPolIn = 1 for normalization)
        else if (pol[0] != 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 && pol[4] != 9) {
          nPolIn = 1;
          HEL[1] = (int)(1 + (pol[0] + 1)*16 + (pol[4] + 1)/2 + (pol[3] + 1) 
                         + 2*(pol[2] + 1) + 4*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (abs(id[1]) % 2 == 0) swapZud();
        if (nGluonTot == 2) { // q g g qbar
          coupl_mass_.dmass = m[1];
          sz2dggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 4, 2, 3]
        }
        else if (nGluonTot == 0) { // q qbar' q' qbar
          coupl_mass_.dmass = m[1];
          coupl_mass_.umass = m[2];
          coupl_mass_.smass = m[2];
          // Check if all quarks have the same flavour / same type (u/d)
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) ) sameFlav = true;
          bool sameType = false;
          if ( (abs(id[1])%2==1) && (abs(id[2])%2==1) && (abs(id[3])%2==1) && (abs(id[4])%2==1) ) sameType = true;
          if ( (abs(id[1])%2==0) && (abs(id[2])%2==0) && (abs(id[3])%2==0) && (abs(id[4])%2==0) ) sameType = true;
               if ( (!sameFlav) && (!sameType) ) sz2duxudx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
          else if ( (!sameFlav) &&  (sameType) ) sz2dsxsdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
          else if (  (sameFlav) &&  (sameType) ) sz2ddxddx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
        }
        if (abs(id[1]) % 2 == 0) swapZud();
      }
      
      // Z->jjjjj
      else if (nOut == 5) {
        // 0) Default assumption: fully unpolarized Z decay
        // 1) Sum over Z polarizations, for specific final-state polarizations
        if (pol[0] == 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
            && pol[4] != 9 && pol[5] != 9) {
          HEL[1] = (int)(1 + (pol[5] + 1)/2 + (pol[4] + 1) + 2*(pol[3] + 1) 
                         + 4*(pol[2] + 1) + 8*(pol[1] + 1));
          HEL[2] = (int)(33 + (pol[5] + 1)/2 + (pol[4] + 1) + 2*(pol[3] + 1) 
                         + 4*(pol[2] + 1) + 8*(pol[1] + 1));
          HEL[3] = (int)(65 + (pol[5] + 1)/2 + (pol[4] + 1) + 2*(pol[3] + 1) 
                         + 4*(pol[2] + 1) + 8*(pol[1] + 1));
        }
        // 2) Fully polarized Z decay (-> nPolIn = 1 for normalization)
        else if (pol[0] != 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
                 && pol[4] != 9 && pol[5] != 9) {
          nPolIn = 1;
          HEL[1] = (int)(1 + (pol[0] + 1)*32 + (pol[5] + 1)/2 + (pol[4] + 1) 
                         + 2*(pol[3] + 1) + 4*(pol[2] + 1) + 8*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
          
        // Convert momenta to MadGraph format
        double PNEW[6][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (abs(id[1]) % 2 == 0) swapZud();
        if (nGluonTot == 3) { // q g g g qbar
          coupl_mass_.dmass = m[1];
          sz2dgggdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 5, 2, 3, 4]
        }
        else if (nGluonTot == 1) { // q qbar' q' g qbar
          coupl_mass_.dmass = m[1];
          coupl_mass_.umass = m[2];
          coupl_mass_.smass = m[2];
          // Check if all quarks have the same flavour / same type (u/d)
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[5])) ) sameFlav = true;
          bool sameType = false;
          if ( (abs(id[1])%2==1) && (abs(id[2])%2==1) && (abs(id[3])%2==1) && (abs(id[5])%2==1) ) sameType = true;
          if ( (abs(id[1])%2==0) && (abs(id[2])%2==0) && (abs(id[3])%2==0) && (abs(id[5])%2==0) ) sameType = true;
               if ( (!sameFlav) && (!sameType) ) sz2duxugdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          else if ( (!sameFlav) &&  (sameType) ) sz2dsxsgdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          else if (  (sameFlav) &&  (sameType) ) sz2ddxdgdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
        }
        if (abs(id[1]) % 2 == 0) swapZud();
      }
      
      // Z->jjjjjj
      else if (nOut == 6) {
        // 0) Default assumption: fully unpolarized Z decay
        // 1) Sum over Z polarizations, for specific final-state polarizations
        if (pol[0] == 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
            && pol[4] != 9 && pol[5] != 9 && pol[6] != 9) {
          HEL[1] = (int)(1 + (pol[6] + 1)/2 + (pol[5] + 1) + 2*(pol[4] + 1) 
                         + 4*(pol[3] + 1) + 8*(pol[2] + 1) + 16*(pol[1] + 1));
          HEL[2] = (int)(65 + (pol[6] + 1)/2 + (pol[5] + 1) + 2*(pol[4] + 1) 
                         + 4*(pol[3] + 1) + 8*(pol[2] + 1) + 16*(pol[1] + 1));
          HEL[3] = (int)(129 + (pol[6] + 1)/2 + (pol[5] + 1) + 2*(pol[4] + 1) 
                         + 4*(pol[3] + 1) + 8*(pol[2] + 1) + 16*(pol[1] + 1));
        }
        // 2) Fully polarized Z decay (-> nPolIn = 1 for normalization)
        else if (pol[0] != 9 && pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
                 && pol[4] != 9 && pol[5] != 9 && pol[6] != 9) {
          nPolIn = 1;
          HEL[1] = (int)(1 + (pol[0] + 1)*64 + (pol[6] + 1)/2 + (pol[5] + 1) 
                         + 2*(pol[4] + 1) + 4*(pol[3] + 1) + 8*(pol[2] + 1) 
                         + 16*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        int icol1  = 1; // different color flows
        int icol5  = 5;
        int icol7  = 7;
        int icol11 = 11;
        // Convert momenta to MadGraph format
        double PNEW[7][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 4) { // q g g g qbar
          coupl_mass_.dmass = m[1];
          if (abs(id[1]) % 2 == 0) swapZud();
          sz2dggggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 6, 2, 3, 4, 5]
          if (abs(id[1]) % 2 == 0) swapZud();
        }
        else if (nGluonTot == 2) { // q qbar' q' g g qbar  and  q g qbar' q' g qbar
          if (abs(id[1]) % 2 == 0) swapZud();
          if ( (id[4]==21) && (id[5]==21) ) { // q qbar' q' g g qbar
            coupl_mass_.dmass = m[1];
            coupl_mass_.umass = m[2];
            coupl_mass_.smass = m[2];
            // Check if all quarks have the same flavour / same type (u/d)
            bool sameFlav = false;
            if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[6])) ) sameFlav = true;
            bool sameType = false;
            if ( (abs(id[1])%2==1) && (abs(id[2])%2==1) && (abs(id[3])%2==1) && (abs(id[6])%2==1) ) sameType = true;
            if ( (abs(id[1])%2==0) && (abs(id[2])%2==0) && (abs(id[3])%2==0) && (abs(id[6])%2==0) ) sameType = true;
                 if ( (!sameFlav) && (!sameType) ) sz2duxuggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            else if ( (!sameFlav) &&  (sameType) ) sz2dsxsggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            else if (  (sameFlav) &&  (sameType) ) sz2ddxdggdx_(&colorDepth,&icol7,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
          }
          else if ( (id[2]==21) && (id[5]==21) ) { // q g qbar' q' g qbar
            // Swap momenta: 2->4, 3->2, 4->3           |         |
            //                                          -----------
            coupl_mass_.dmass = m[1];
            coupl_mass_.umass = m[3];
            coupl_mass_.smass = m[3];
            double PTMP[7][4];
            for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
            for (int i=0; i<4; ++i) {
              PNEW[2][i] = PTMP[3][i];
              PNEW[3][i] = PTMP[4][i];
              PNEW[4][i] = PTMP[2][i];
            }
            // Check if all quarks have the same flavour / same type (u/d)
            bool sameFlav = false;
            if ( (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) && (abs(id[1])==abs(id[6])) ) sameFlav = true;
            bool sameType = false;
            if ( (abs(id[1])%2==1) && (abs(id[3])%2==1) && (abs(id[4])%2==1) && (abs(id[6])%2==1) ) sameType = true;
            if ( (abs(id[1])%2==0) && (abs(id[3])%2==0) && (abs(id[4])%2==0) && (abs(id[6])%2==0) ) sameType = true;
                 if ( (!sameFlav) && (!sameType) ) sz2duxuggdx_(&colorDepth,&icol5 ,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            else if ( (!sameFlav) &&  (sameType) ) sz2dsxsggdx_(&colorDepth,&icol5 ,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            else if (  (sameFlav) &&  (sameType) ) sz2ddxdggdx_(&colorDepth,&icol11,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
          }
          if (abs(id[1]) % 2 == 0) swapZud();
        }
        else if (nGluonTot == 0) { // q qbar' q' qbar'' q'' qbar
          vector<int> downType;
          vector<int> upType;
          map<int,int> flavours;
          for (int i=1; i<=5; ++i) flavours[i] = 0;
          for (int i=1; i<nMom; ++i) {
            flavours[abs(id[i])]++;
            if (abs(id[i]) % 2 == 0) upType.push_back(i); else downType.push_back(i);
          }
          int countSameFlavour = 0; // 0 if 0 or 1 quark pair, 1 if 2 quark pairs, 2 if 3 quark pais
          for (int i=1; i<=5; ++i) countSameFlavour = max( max((flavours[i]-2)/2,0) , countSameFlavour);
          // 2 down type quark pairs and 1 up type quark pair or
          // 1 down type quark pair and 2 up type quark pairs
          if ( (downType.size()==4 && upType.size()==2) ||
               (downType.size()==2 && upType.size()==4) ) {
            double PTMP[7][4];
            for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
            if (countSameFlavour == 1) {
              // flavour that appears once
              int absFlav = 0;
              for (int i=1; i<=5; ++i) if (flavours[i] == 2) absFlav = i;
              pair<int,int> flavourPos; // quark first, antiquark second
              for (int i=1; i<nMom; ++i) { if (abs(id[i]) == absFlav) {
                if (id[i] > 0) flavourPos.first = i; else flavourPos.second = i;
              } }
              vector<int> order = orderQQBqqbqqb(flavourPos);
              for (int i=0; i<nMom; ++i) {
                for (int j=0; j<4; ++j) {
                  PNEW[i][j] = PTMP[order[i]][j];
                }
              }
              coupl_mass_.dmass = m[order[1]];
              coupl_mass_.umass = m[order[2]];
              if (abs(id[order[1]]) % 2 == 0) swapZud();
              sz2duxuuxudx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              if (abs(id[order[1]]) % 2 == 0) swapZud();
            }
            else {
              // type that appears once
              int type = ( (downType.size()==2) ? 1 : 0 );
              pair<int,int> typePos; // quark first, antiquark second
              for (int i=1; i<nMom; ++i) { if (abs(id[i]) % 2 == type) {
                if (id[i] > 0) typePos.first = i; else typePos.second = i;
              } }
              vector<int> order = orderQQBqqbqqb(typePos, false);
              for (int i=0; i<nMom; ++i) {
                for (int j=0; j<4; ++j) {
                  PNEW[i][j] = PTMP[order[i]][j];
                }
              }
              coupl_mass_.dmass = m[order[1]];
              coupl_mass_.smass = m[order[2]];
              coupl_mass_.umass = m[order[4]];
              if (abs(id[order[1]]) % 2 == 0) swapZud();
              sz2dsxsuxudx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              if (abs(id[order[1]]) % 2 == 0) swapZud();
            }
          }
          // 3 down type quark pairs or 3 up type quark pairs
          else {
            if (countSameFlavour == 2) {
              coupl_mass_.dmass = m[1];
              if (abs(id[1]) % 2 == 0) swapZud();
              sz2ddxddxddx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              if (abs(id[1]) % 2 == 0) swapZud();
            }
            else if (countSameFlavour == 1) {
              // flavour that appears once
              int absFlav = 0;
              for (int i=1; i<=5; ++i) if (flavours[i] == 2) absFlav = i;
              pair<int,int> flavourPos; // quark first, antiquark second
              for (int i=1; i<nMom; ++i) { if (abs(id[i]) == absFlav) {
                if (id[i] > 0) flavourPos.first = i; else flavourPos.second = i;
              } }
              vector<int> order = orderQQBqqbqqb(flavourPos);
              double PTMP[7][4];
              for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
              for (int i=0; i<nMom; ++i) {
                for (int j=0; j<4; ++j) {
                  PNEW[i][j] = PTMP[order[i]][j];
                }
              }
              coupl_mass_.dmass = m[order[1]];
              coupl_mass_.smass = m[order[2]];
              if (abs(id[order[1]]) % 2 == 0) swapZud();
              sz2dsxssxsdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              if (abs(id[order[1]]) % 2 == 0) swapZud();
            }
            else if (countSameFlavour == 0) {
              // pick one flavour to make decision
              int absFlav = 0;
              for (int i=1; i<=5; ++i) if (flavours[i] == 2) absFlav = i;
              pair<int,int> flavourPos; // quark first, antiquark second
              for (int i=1; i<nMom; ++i) { if (abs(id[i]) == absFlav) {
                if (id[i] > 0) flavourPos.first = i; else flavourPos.second = i;
              } }
              vector<int> order = orderQQBqqbqqb(flavourPos);
              double PTMP[7][4];
              for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
              for (int i=0; i<nMom; ++i) {
                for (int j=0; j<4; ++j) {
                  PNEW[i][j] = PTMP[order[i]][j];
                }
              }
              coupl_mass_.dmass = m[order[1]];
              coupl_mass_.smass = m[order[2]];
              coupl_mass_.bmass = m[order[4]];
              if (abs(id[1]) % 2 == 0) swapZud();
              sz2dsxsbxbdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              if (abs(id[1]) % 2 == 0) swapZud();
            }
          }
        }
      }
      
    }
    // Z decay
    //------------------------------------------------------------------------
    
    //------------------------------------------------------------------------
    // W decay, unpolarized only
    // TODO for polarized we need to distinguish between W+ and W-
    else if (nIn == 1 && id0 == 24) {
      
      // In case of W- reverse order of quarks and use -ids
      if (id[0] < 0) {
        vector<int>  idsTmp = id;
        vector<Vec4> psTmp  = p;
        if (nOut == 5 && nGluonTot == 1) { // Special case 1
          p[1]  = psTmp[2]; 
          id[1] = -idsTmp[2];
          p[2]  = psTmp[1]; 
          id[2] = -idsTmp[1];
          p[3]  = psTmp[5]; 
          id[3] = -idsTmp[5];
          p[5]  = psTmp[3]; 
          id[5] = -idsTmp[3];
        } else if (nOut == 6 && nGluonTot == 2 && 
                   id[4] == 21 && id[5] == 21) { // Special case 2
          p[1]  = psTmp[2]; 
          id[1] = -idsTmp[2];
          p[2]  = psTmp[1]; 
          id[2] = -idsTmp[1];
          p[3]  = psTmp[6]; 
          id[3] = -idsTmp[6];
          p[4]  = psTmp[5]; 
          p[5]  = psTmp[4]; 
          p[6]  = psTmp[3]; 
          id[6] = -idsTmp[3];
        } else {
          // Loop over all partons
          for (int i=1; i<nMom; ++i) {
            p[i]  = psTmp[nMom-i];
            id[i] = (idsTmp[nMom-i] == 21 ? 21 : -idsTmp[nMom-i]);
          }
        }
      }

      // Ensure ME evaluated on shell
      const double mSys = p[0].mCalc();
      coupl_mass_.wmass = mSys;
      
      // W->jj
      if (nOut == 2) { if ((id[1]!=21) && (id[2]!=21)) {
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        coupl_mass_.umass = m[1];
        coupl_mass_.dmass = m[2];
        setWcouplings(abs(id[1]), abs(id[2]));
        swx2udx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
      } }
      
      // W->jjj
      else if (nOut == 3) {
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        coupl_mass_.umass = m[1];
        coupl_mass_.dmass = m[3];
        setWcouplings(abs(id[1]), abs(id[3]));
        swx2ugdx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 3, 2]
      }
      
      // W->jjjj
      else if (nOut == 4) {
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 2) { // q g g qbar
          coupl_mass_.umass = m[1];
          coupl_mass_.dmass = m[4];
          setWcouplings(abs(id[1]), abs(id[4]));
          swx2uggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 4, 2, 3]
        }
        else if (nGluonTot == 0) { // q qbar q qbar
          // Sort such that the quark coupling to W goes first
          vector<int> order = orderWfourQuarks(id);
          coupl_mass_.umass = m[order[1]];
          coupl_mass_.dmass = m[order[4]];
          setWcouplings(abs(id[order[1]]), abs(id[order[4]]));
          double PTMP[5][4];
          for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
          for (int i=0; i<nMom; ++i) {
            for (int j=0; j<4; ++j) {
              PNEW[i][j] = PTMP[order[i]][j];
            }
          }
          if (id[order[1]] == id[order[3]]) { // q qbar q qbar'
            swx2uuxudx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
          } else if (id[order[2]] == id[order[4]]) { // q' qbar q qbar
            swx2udxddx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
          } else { // q' qbar q qbar''
            coupl_mass_.smass = m[order[2]];
            swx2usxsdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]
          }
        }
      }
      
      // W->jjjjj
      else if (nOut == 5) {
        int icol1 = 1; // different color flows
        int icol2 = 2;
        int icol3 = 3;
        int icol4 = 4;
        // Convert momenta to MadGraph format
        double PNEW[6][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 3) { // q g g g qbar
          coupl_mass_.umass = m[1];
          coupl_mass_.dmass = m[5];
          setWcouplings(abs(id[1]), abs(id[5]));
          swx2ugggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 5, 2, 3, 4]
        }
        else if (nGluonTot == 1) { // q qbar q g qbar
          // Sort such that the quark coupling to W goes first
          bool gSwap;
          vector<int> order = orderWfourQuarksG(id, gSwap);
          coupl_mass_.umass = m[order[1]];
          coupl_mass_.dmass = m[order[5]];
          setWcouplings(abs(id[order[1]]), abs(id[order[5]]));
          double PTMP[6][4];
          for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
          for (int i=0; i<nMom; ++i) {
            for (int j=0; j<4; ++j) {
              PNEW[i][j] = PTMP[order[i]][j];
            }
          }
          if (id[order[1]] == id[order[3]]) { // q qbar q g qbar'
            if (gSwap) swx2uuxugdx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 3, 5]T[ 1, 2, 4]
            else       swx2uuxugdx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          } else if (id[order[2]] == id[order[5]]) { // q' qbar q g qbar
            if (gSwap) swx2udxdgdx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 3, 5]T[ 1, 2, 4]
            else       swx2udxdgdx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          } else { // q' qbar q g qbar''
            coupl_mass_.smass = m[order[2]];
            if (gSwap) swx2usxsgdx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 5]T[ 1, 2, 4]
            else       swx2usxsgdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          }
        }
      }
      
      // W->jjjjjj
      else if (nOut == 6) {
        int icol1  = 1; // different color flows
        int icol2  = 2;
        int icol3  = 3;
        int icol4  = 4;
        int icol6  = 6;
        int icol7  = 7;
        int icol10 = 10;
        int icol12 = 12;
        // Convert momenta to MadGraph format
        double PNEW[7][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 4) { // q g g g g qbar
          coupl_mass_.umass = m[1];
          coupl_mass_.dmass = m[6];
          setWcouplings(abs(id[1]), abs(id[6]));
          swx2uggggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 6, 2, 3, 4, 5]
        }
        else if (nGluonTot == 2) { // q qbar q g g qbar  and  q g qbar q g qbar
          // Sort such that the quark coupling to W goes first
          bool gg;
          bool gSwap;
          vector<int> order = orderWfourQuarksGG(id, gg, gSwap);
          coupl_mass_.umass = m[order[1]];
          coupl_mass_.dmass = m[order[6]];
          setWcouplings(abs(id[order[1]]), abs(id[order[6]]));
          double PTMP[7][4];
          for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
          for (int i=0; i<nMom; ++i) {
            for (int j=0; j<4; ++j) {
              PNEW[i][j] = PTMP[order[i]][j];
            }
          }
          if (id[order[1]] == id[order[3]]) { // q qbar q g g qbar'
            if (gg) {
              if (gSwap) swx2uuxuggdx_(&colorDepth,&icol12,PNEW,HEL,&MGME2); // T[ 3, 6]T[ 1, 2, 4, 5]
              else       swx2uuxuggdx_(&colorDepth,&icol7 ,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            } else {
              swx2uuxuggdx_(&colorDepth,&icol10,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            }
          } else if (id[order[2]] == id[order[6]]) { // q' qbar q g g qbar
            if (gg) {
              if (gSwap) swx2udxdggdx_(&colorDepth,&icol12,PNEW,HEL,&MGME2); // T[ 3, 6]T[ 1, 2, 4, 5]
              else       swx2udxdggdx_(&colorDepth,&icol7 ,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            } else {
              swx2udxdggdx_(&colorDepth,&icol10,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            }
          } else { // q' qbar q g g qbar''
            coupl_mass_.smass = m[order[2]];
            if (gg) {
              if (gSwap) swx2usxsggdx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 3, 6]T[ 1, 2, 4, 5]
              else       swx2usxsggdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            } else {
              swx2usxsggdx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            }
          }
        }
        else if (nGluonTot == 0) { // q qbar q qbar q qbar
          // Sort such that the quark coupling to W goes first
          map<int,int> flavours;
          bool flav2ordChange = false;
          vector<int> order = orderWsixQuarks(id, flavours, flav2ordChange);
          coupl_mass_.umass = m[order[1]];
          coupl_mass_.dmass = m[order[6]];
          setWcouplings(abs(id[order[1]]), abs(id[order[6]]));
          double PTMP[7][4];
          for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
          for (int i=0; i<nMom; ++i) {
            for (int j=0; j<4; ++j) {
              PNEW[i][j] = PTMP[order[i]][j];
            }
          }
          int flavCount = 0;
          for (int i=1; i<=5; ++i) if (flavours[i] > 0) flavCount++;
          if (flavCount == 4) { // q'' qbar q qbar' q' qbar'''
            coupl_mass_.smass = m[order[2]];
            coupl_mass_.cmass = m[order[4]];
            swx2usxscxcdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
          } else if (flavCount == 3) {
            if (flavours[abs(id[order[1]])] > 1) {
              if ( (abs(id[order[1]]) == abs(id[order[2]])) &&
                   (abs(id[order[1]]) == abs(id[order[3]])) ) { // q qbar q qbar' q' qbar''
                coupl_mass_.smass = m[order[4]];
                swx2uuxusxsdx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              }
              else if ( (abs(id[order[1]]) == abs(id[order[3]])) &&
                        (abs(id[order[1]]) == abs(id[order[4]])) ) { // q qbar' q qbar q' qbar''
                // Swap 12 with 34 -> q qbar q qbar' q' qbar''
                for (int j=0; j<4; ++j) {
                  PNEW[1][j] = PTMP[order[3]][j];
                  PNEW[2][j] = PTMP[order[4]][j];
                  PNEW[3][j] = PTMP[order[1]][j];
                  PNEW[4][j] = PTMP[order[2]][j];
                }
                coupl_mass_.umass = m[order[3]];
                setWcouplings(abs(id[order[3]]), abs(id[order[6]]));
                coupl_mass_.smass = m[order[2]];
                swx2uuxusxsdx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              }
              else if ( (abs(id[order[1]]) == abs(id[order[4]])) &&
                        (abs(id[order[1]]) == abs(id[order[5]])) ) { // q qbar' q' qbar q qbar''
                // Swap 23 with 45 -> q qbar q qbar' q' qbar''
                for (int j=0; j<4; ++j) {
                  PNEW[2][j] = PTMP[order[4]][j];
                  PNEW[3][j] = PTMP[order[5]][j];
                  PNEW[4][j] = PTMP[order[2]][j];
                  PNEW[5][j] = PTMP[order[3]][j];
                }
                coupl_mass_.smass = m[order[2]];
                swx2uuxusxsdx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 1, 4]T[ 5, 2]T[ 3, 6]
              }
              else if (verbose >= 3) {
                printOut("MG4interface::ME","sorting problem in W -> 6q with 3 different flavours"
                                            " and #flav[1] > 1)");
                for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
                cout << endl;
                for (int i=0; i<(int)id.size(); i++) cout << id[order[i]] << " ";
                cout << endl;
              }
            } else if (flavours[abs(id[order[6]])] > 1) {
              if ( (abs(id[order[6]]) == abs(id[order[2]])) &&
                   (abs(id[order[6]]) == abs(id[order[3]])) ) { // q'' qbar q qbar' q' qbar
                coupl_mass_.smass = m[order[4]];
                swx2udxdsxsdx_(&colorDepth,&icol4,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
              }
              else if ( (abs(id[order[6]]) == abs(id[order[3]])) &&
                        (abs(id[order[6]]) == abs(id[order[4]])) ) { // q'' qbar' q qbar q' qbar
                // Swap 2 with 4 -> q'' qbar q qbar' q' qbar
                for (int j=0; j<4; ++j) {
                  PNEW[2][j] = PTMP[order[4]][j];
                  PNEW[4][j] = PTMP[order[2]][j];
                }
                coupl_mass_.smass = m[order[2]];
                swx2udxdsxsdx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 4]T[ 3, 2]T[ 5, 6]
              }
              else if ( (abs(id[order[6]]) == abs(id[order[4]])) &&
                        (abs(id[order[6]]) == abs(id[order[5]])) ) { // q'' qbar' q' qbar q qbar
                // Swap 23 with 45 -> q'' qbar q qbar' q' qbar
                for (int j=0; j<4; ++j) {
                  PNEW[2][j] = PTMP[order[4]][j];
                  PNEW[3][j] = PTMP[order[5]][j];
                  PNEW[4][j] = PTMP[order[2]][j];
                  PNEW[5][j] = PTMP[order[3]][j];
                }
                coupl_mass_.smass = m[order[2]];
                swx2udxdsxsdx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 4]T[ 5, 2]T[ 3, 6]
              }
              else if (verbose >= 3) {
                printOut("MG4interface::ME","sorting problem in W -> 6q with 3 different flavours"
                                            " and #flav[6] > 1)");
                for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
                cout << endl;
                for (int i=0; i<(int)id.size(); i++) cout << id[order[i]] << " ";
                cout << endl;
              }
            } else if ( (abs(id[order[2]]) == abs(id[order[3]])) &&
                        (abs(id[order[2]]) == abs(id[order[4]]))  &&
                        (abs(id[order[2]]) == abs(id[order[5]])) ) { // q' qbar q qbar q qbar''
              coupl_mass_.smass = m[order[4]];
              swx2usxssxsdx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
            }
            else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in W -> 6q with 3 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
              for (int i=0; i<(int)id.size(); i++) cout << id[order[i]] << " ";
              cout << endl;
            }
          } else {
            if ( (abs(id[order[1]]) == abs(id[order[2]])) &&
                 (abs(id[order[1]]) == abs(id[order[3]])) &&
                 (abs(id[order[1]]) == abs(id[order[4]])) &&
                 (abs(id[order[1]]) == abs(id[order[5]])) ) // q qbar q qbar q qbar'
              swx2uuxuuxudx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
            else if ( (abs(id[order[6]]) == abs(id[order[2]])) &&
                      (abs(id[order[6]]) == abs(id[order[3]])) &&
                      (abs(id[order[6]]) == abs(id[order[4]])) &&
                      (abs(id[order[6]]) == abs(id[order[5]])) ) // q' qbar q qbar q qbar
              swx2udxddxddx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
            else if ( (abs(id[order[1]]) == abs(id[order[2]])) &&
                      (abs(id[order[1]]) == abs(id[order[3]])) ) { // q qbar q qbar' q' qbar'
              if (flav2ordChange) swx2uuxudxddx_(&colorDepth,&icol3,PNEW,HEL,&MGME2); // T[ 1, 4]T[ 5, 2]T[ 3, 6]
              else swx2uuxudxddx_(&colorDepth,&icol6,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
            }
            else if (verbose >= 3) {
              printOut("MG4interface::ME","sorting problem in W -> 6q with 2 different flavours");
              for (int i=0; i<(int)id.size(); i++) cout << id[i] << " ";
              cout << endl;
              for (int i=0; i<(int)id.size(); i++) cout << id[order[i]] << " ";
              cout << endl;
            }
          }
        }
      }
      
    }
    // W decay
    //------------------------------------------------------------------------
    
    //------------------------------------------------------------------------
    // H decay
    // NOTE If the quarks are massless, the pre-calculated Yukawa couplings
    // are used. If they are massive, the coupling gets re-calculated
    else if (nIn == 1 && id0 == 25) {
      
      // Polarized?
      if (isPolarised) HEL[0] = 1;
      nPolIn = 1; // Default: sum over all incoming and outgoing polarizations

      // Ensure ME evaluated on shell
      const double mSys = p[0].mCalc();
      coupl_mass_.hmass = mSys;
      
      // H->jj
      if (nOut == 2) {
        // 0) Default assumption: fully unpolarized H decay
        // 1) Fully polarized H decay
        if (pol[1] != 9 && pol[2] != 9) {
          HEL[1] = (int)(1 + (pol[2] + 1)/2 + (pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        // Convert momenta to MadGraph format
        double PNEW[3][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 2) sh2gg_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        else {
          setYukawa(abs(id[1]), m[1]);
          sh2bbx_(&colorDepth,&icol,PNEW,HEL,&MGME2);
        }
      }
      
      // H->jjj
      else if (nOut == 3) {
        // 0) Default assumption: fully unpolarized H decay
        // 1) Fully polarized H decay
        if (pol[1] != 9 && pol[2] != 9 && pol[3] != 9) {
          HEL[1] = (int)(1 + (pol[3] + 1)/2 + (pol[2] + 1) 
                         + 2*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        // Convert momenta to MadGraph format
        double PNEW[4][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 3) sh2ggg_(&colorDepth,&icol,PNEW,HEL,&MGME2); // F[1, 2, 3]
        else {
          setYukawa(abs(id[1]), m[1]);
          sh2bgbx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[1, 3, 2]
        }
      }
      
      // H->jjjj
      else if (nOut == 4) {
        // 0) Default assumption: fully unpolarized H decay
        // 1) Fully polarized H decay
        if (pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
                 && pol[4] != 9) {
          HEL[1] = (int)(1 + (pol[4] + 1)/2 + (pol[3] + 1) 
                         + 2*(pol[2] + 1) + 4*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        int icol1 = 1; // different color flows
        int icol2 = 2;
        // Convert momenta to MadGraph format
        double PNEW[5][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 4) sh2gggg_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // F[1, 2, 3, 4]
        else if (nGluonTot == 2) { // q g g qbar
          setYukawa(abs(id[1]), m[1]);
          sh2bggbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[1, 4, 2, 3]
        }
        else if (nGluonTot == 0) { // q qbar' q' qbar
          // Check if all quarks have the same flavour
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) ) sameFlav = true;
          setYukawa(abs(id[1]), abs(id[2]), m[1], m[2]);
          if (sameFlav) sh2bbxbbx_(&colorDepth,&icol2,PNEW,HEL,&MGME2); // T[ 3, 4]T[ 1, 2]
          else          sh2bcxcbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 3, 4]T[ 1, 2]
        }
      }
      
      // H->jjjjj
      else if (nOut == 5) {
        // 0) Default assumption: fully unpolarized H decay
        // 1) Fully polarized H decay
        if (pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
                 && pol[4] != 9 && pol[5] != 9) {
          HEL[1] = (int)(1 + (pol[5] + 1)/2 + (pol[4] + 1) 
                         + 2*(pol[3] + 1) + 4*(pol[2] + 1) + 8*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        // Convert momenta to MadGraph format
        double PNEW[6][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 5) sh2ggggg_(&colorDepth,&icol,PNEW,HEL,&MGME2); // F[1, 2, 3, 4, 5]
        else if (nGluonTot == 3) { // q g g g qbar
          setYukawa(abs(id[1]), m[1]);
          sh2bgggbx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 5, 2, 3, 4]
        }
        else if (nGluonTot == 1) { // q qbar' q' g qbar
          // Check if all quarks have the same flavour
          bool sameFlav = false;
          if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[5])) ) sameFlav = true;
          setYukawa(abs(id[1]), abs(id[2]), m[1], m[2]);
          if (sameFlav) sh2bbxbgbx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
          else          sh2bcxcgbx_(&colorDepth,&icol,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 5, 4]
        }
      }
      
      // H->jjjjjj
      else if (nOut == 6) {
        // 0) Default assumption: fully unpolarized H decay
        // 1) Fully polarized H decay
        if (pol[1] != 9 && pol[2] != 9 && pol[3] != 9 
            && pol[4] != 9 && pol[5] != 9 && pol[6] != 9) {
          HEL[1] = (int)(1 + (pol[6] + 1)/2 + (pol[5] + 1) 
                         + 2*(pol[4] + 1) + 4*(pol[3] + 1) + 8*(pol[2] + 1) 
                         + 16*(pol[1] + 1));
          HEL[2] = -1;
          HEL[3] = -1;
        }
        
        int icol1  = 1; // different color flows
        int icol5  = 5;
        int icol7  = 7;
        int icol9  = 9;
        int icol10 = 10;
        // Convert momenta to MadGraph format
        double PNEW[7][4];
        for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PNEW[i]);
        if (nGluonTot == 6) sh2gggggg_(&colorDepth,&icol9,PNEW,HEL,&MGME2); // F[1, 2, 3, 4, 5, 6]
        else if (nGluonTot == 4) { // q g g g g qbar
          setYukawa(abs(id[1]), m[1]);
          sh2bggggbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 6, 2, 3, 4, 5]
        }
        else if (nGluonTot == 2) { // q qbar' q' g g qbar  and  q g qbar' q' g qbar
          if ( (id[4]==21) && (id[5]==21) ) { // q qbar' q' g g qbar
            // Check if all quarks have the same flavour
            bool sameFlav = false;
            if ( (abs(id[1])==abs(id[2])) && (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[6])) ) sameFlav = true;
            setYukawa(abs(id[1]), abs(id[2]), m[1], m[2]);
            if (sameFlav) sh2bbxbggbx_(&colorDepth,&icol7,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
            else          sh2bcxcggbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 6, 4, 5]
          }
          else if ( (id[2]==21) && (id[5]==21) ) { // q g qbar' q' g qbar
            // Swap momenta: 2->4, 3->2, 4->3           |         |
            //                                          -----------
            double PTMP[7][4];
            for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
            for (int i=0; i<4; ++i) {
              PNEW[2][i] = PTMP[3][i];
              PNEW[3][i] = PTMP[4][i];
              PNEW[4][i] = PTMP[2][i];
            }
            // Check if all quarks have the same flavour
            bool sameFlav = false;
            if ( (abs(id[1])==abs(id[3])) && (abs(id[1])==abs(id[4])) && (abs(id[1])==abs(id[6])) ) sameFlav = true;
            setYukawa(abs(id[1]), abs(id[3]), m[1], m[3]);
            if (sameFlav) sh2bbxbggbx_(&colorDepth,&icol10,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
            else          sh2bcxcggbx_(&colorDepth,&icol5 ,PNEW,HEL,&MGME2); // T[ 1, 2, 4]T[ 3, 6, 5]
          }
        }
        else if (nGluonTot == 0) { // q qbar' q' qbar'' q'' qbar
          map<int,int> flavours;
          for (int i=1; i<=5; ++i) flavours[i] = 0;
          for (int i=1; i<nMom; ++i) flavours[abs(id[i])]++;
          int countSameFlavour = 0; // 0 if 0 or 1 quark pair, 1 if 2 quark pairs, 2 if 3 quark pais
          for (int i=1; i<=5; ++i) countSameFlavour = max( max((flavours[i]-2)/2,0) , countSameFlavour);
          if (countSameFlavour==2) {
            setYukawa(abs(id[1]), 0, m[1], 0.0);
            sh2bbxbbxbbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
          }
          else if (countSameFlavour==1) {
            // flavour that appears once
            int absFlav = 0;
            for (int i=1; i<=5; ++i) if (flavours[i] == 2) absFlav = i;
            pair<int,int> flavourPos; // quark first, antiquark second
            for (int i=1; i<nMom; ++i) { if (abs(id[i]) == absFlav) {
              if (id[i] > 0) flavourPos.first = i; else flavourPos.second = i;
            } }
            vector<int> order = orderQQBqqbqqb(flavourPos);
            double PTMP[7][4];
            for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
            for (int i=0; i<nMom; ++i) {
              for (int j=0; j<4; ++j) {
                PNEW[i][j] = PTMP[order[i]][j];
              }
            }
            setYukawa(abs(id[order[1]]), abs(id[order[2]]), m[order[1]], m[order[2]]);
            sh2bcxccxcbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
          }
          else if (countSameFlavour==0) {
            // pick one quark pair at ramdom for order, use first quark
            pair<int,int> randomPos; // quark first, antiquark second
            randomPos.first = 1;
            for (int i=2; i<nMom; ++i) if (abs(id[i]) == abs(id[1]))
              randomPos.second = i;
            vector<int> order = orderQQBqqbqqb(randomPos);
            double PTMP[7][4];
            for (int i=0; i<nMom; ++i) Vec4toMG(p[i],PTMP[i]);
            for (int i=0; i<nMom; ++i) {
              for (int j=0; j<4; ++j) {
                PNEW[i][j] = PTMP[order[i]][j];
              }
            }
            setYukawa(abs(id[order[1]]), abs(id[order[2]]), abs(id[order[4]]), 
                      m[order[1]],       m[order[2]],       m[order[4]]);
            coupl_mass_.umass = m[order[4]];
            sh2bcxcsxsbx_(&colorDepth,&icol1,PNEW,HEL,&MGME2); // T[ 1, 2]T[ 3, 4]T[ 5, 6]
          }
        }
      }
      
    }
    // H decay
    //------------------------------------------------------------------------

    if (isnan(MGME2) && verbose >= 1) {
      printOut("MG4interface::ME2","Error! ME2 = nan. Printing momenta:");
      for (int i=0; i<int(p.size()); ++i) {
        cout<<"   p"<<i<<p[i];
      }
    }

    // Return |ME|^2 value (or -1 if no ME found)
    if (abs(MGME2+1) < TINY) return -1.;
    else return MGME2/max(1,nPolIn);

  }

}
